function [TestRes] = analyzeall(varargin)
    % ANALYZEALL Analyzes of a testcases
    %   TODO
    % Note:
    %   - The inertia/added mass tests give strange results. Instead of using these as the baseline
    %     for interial forces, we directly use measurement @ the correct flapping frequency, but
    %     under wind-off conditions. With that, the added mass effects are not removed, but as the
    %     reduced frequency is small enough it is not a big deal.
    % -----
    %
    % Syntax:
    %   TODO
    %
    % Inputs:
    %   TODO
    % Output:
    %   TODO
    %
    % See also: Periods, TestCase, FileLoader.

    % ----------------------------------------------------------------------------------------------
    % FIXME
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    narginchk(0, 9); % FileId + 4 options with their values

    % --- Cleanup, imports and environment setting
    clearvars -except varargin;
    close all;
    clc;
    setcwdhere(); % Make sure we are always in the proper current working directory
    addpath(genpath('.'));

    % --- Input processing
    % Load files to analyze and parse options
    [processedFiles, autosave, debug, quiet, rejectNoInertia] = parseinputs(varargin{:});

    % Analyze each cleaned-up file
    for thisFile = processedFiles

        % Get file information load it
        [thisFpath, thisFname] = fileparts(thisFile);
        conditionalprint(~quiet || debug, sprintf("Analyzing file %s\n", thisFname));
        conditionalprint(~quiet || debug, "**********************************\n");

        TestCases = loadobjects(thisFile);

        TestRes = TestResult; % Re-initialize Objects for each file

        for iTest = 1:numel(TestCases)
            thisCase = TestCases(iTest);

            TestRes(iTest) = TestResult(thisCase);

            % Skip rejected cases
            if thisCase.rejected
                fprintf(2, ['%2d || Dx = %d cm | V = %0.1f m/s | f = %0.2f Hz => '...
                            'REJECTED TESTCASE\n'], iTest, thisCase.spacing(1) * 100, ...
                        thisCase.airspeed, thisCase.frequency);
                continue
            end

            % Find linked inertia result -> corresponds to the last previous result with
            % the same test conditions but the airspeed of 0
            if TestRes(iTest).TestCond.airspeed ~= 0
                idxInertia = [];
                for i = iTest:-1:1
                    if sametestcond(TestRes(i), TestRes(iTest), false) && ...
                            TestRes(i).TestCond.airspeed == 0
                        idxInertia = i;
                        break
                    end
                end
                if isempty(idxInertia) || TestRes(idxInertia).rejected
                    if rejectNoInertia
                        fprintf(2, ['%2d || Dx = %d cm | V = %0.1f m/s | f = %0.2f Hz => ' ...
                                    'REJECTED BASED ON NO INERTIA in %d\n'], ...
                                iTest, thisCase.spacing(1) * 100, thisCase.airspeed, ...
                                thisCase.frequency, idxInertia);
                        TestRes(iTest).rejected = true;
                        continue
                    end
                end
            end

            fprintf('%2d || %s \n', iTest, TestRes(iTest).strcond('short', 'ideal'));

            % Periods analysis
            TestRes(iTest).splitperiods(); % Split signal into periods (based on angles)

            % Remove inertia before proceeding to analysis
            if TestRes(iTest).TestCond.airspeed ~= 0
                if ~TestRes(idxInertia).rejected
                    fprintf('      Removing inertia using data in %d\n', idxInertia);
                    TestRes(iTest).inertia_idx = idxInertia;
                    TestRes(iTest).rminertia(TestRes(idxInertia));
                else
                    fprintf(2, '      INERTIA NOT REMOVED (no inertia in %d)\n', idxInertia);
                end
            end

            if strcmpi(TestRes(iTest).PeriodsF.motion, 'flapping') && ...
                    strcmpi(TestRes(iTest).PeriodsA.motion, 'flapping')
                TestRes(iTest).getoffsets();   % Phase offset between front and aft for each period
                TestRes(iTest).groupoffsets(); % Group and average cycles of same offset
            end

            % --- Plots
            if debug
                % Figure name
                figTitle = TestRes(iTest).strcond('title', 'meanfreq');

                % Plot raw and average data
                TestRes(iTest).plotraw('angle', figTitle);
                TestRes(iTest).plotavg({'angle', 'Fz', 'Fx'}, figTitle);

                % Plot offsets
                figname = 'Analysis - Offsets';
                close(findobj('type', 'figure', 'name', figname));
                figure('Name', figname);
                sgtitle(figTitle);
                TestRes(iTest).plotoffsets({'Fz', 'Fx'}, 'channel', false);

                pause;
            end

            % Prune basic duplicated stuff from all TestResults
            TestRes(iTest).prune({'baseData', 'periods_base'});

        end

        % --- Saving TestResults
        if autosave
            conditionalprint(~quiet || debug, "Saving all TestCase processed data\n\n\n");

            saveFileName = fullfile(thisFpath, strcat("RESULTS-", thisFname));
            saveobjects(TestRes, saveFileName);
        end

    end

end

% --------------------------------------------------------------------------------------------------
function [fileNames, autosave, debug, quiet, rejectNoInertia] = parseinputs(varargin)
    % PARSEINPUTS Parse function input and return cleaned inputs and options

    % Default and constants
    DATA_DIR = '../data/cleaned'; % Location of the data dir w.r.t. current file
    FILE_EXT =  {'*.mat'}; % Accepted file extensions

    % Validation functions
    valLogi = @(x) validateattributes(x, {'logical'}, {'scalar'}, mfilename());

    % Parse inputs
    p = inputParser;
    addOptional(p, 'files', []);
    addParameter(p, 'autosave', true, valLogi);
    addParameter(p, 'debug', false, valLogi);
    addParameter(p, 'quiet', false, valLogi);
    addParameter(p, 'rejectNoInertia', true, valLogi);
    parse(p, varargin{:});
    tmpFiles = string(p.Results.files);
    autosave = p.Results.autosave;
    debug = p.Results.debug;
    quiet = p.Results.quiet;
    rejectNoInertia = p.Results.rejectNoInertia;

    % Get proper file names
    fileNames = getfiles(DATA_DIR, FILE_EXT, tmpFiles);

end

function bool = sametestcond(TestRes1, TestRes2, airspeedEqual)
    % SAMETESTCOND Check if two tests results have the same test conditions
    %   If airspeedEqual = true, the airspeed must be equal as well. If false, the airspeeds can
    %   be different.

    bool = isequaln(TestRes1.TestCond.spacing, TestRes2.TestCond.spacing) && ...
        TestRes1.TestCond.frequency == TestRes2.TestCond.frequency && ...
        (~airspeedEqual || TestRes1.TestCond.airspeed == TestRes2.TestCond.airspeed);

end
