classdef ArduData < GenericData
    % ARDUDATA Represent a single Arduino Data file
    %   This class loads, clean and store the Arduino Data corresponding to a single test case.
    % -----
    %
    % ArduData properties:
    %
    % ArduData methods:
    %
    % ArduData constructor:
    %   Ardu = ArduData() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: GenericData, TestCase.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % FIXME: (L) Make it so consturctor requests file if nothing is provided
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties
        Raw = []    % Raw data
        Cleaned     % Cleaned and calibrated data
        Trimmed     % Trimmed and resampled data
        filename    % Data filename
        sampling    % Sampling rate, [Hz]
    end

    properties (Access  = protected)
        % We only extract the data we need from the CSV files
        csv_selectedDataNames = {'RXDate_Time', 'x1_Left_field6_', 'x1_Right_field7_', ...
                                 'x2_Left_field8_', 'x2_Right_field9_', 'x1_Tension_field14_', ...
                                 'x1_Current_field15_', 'x2_Tension_field16_', ...
                                 'x2_Current_field17_'}
        csv_firstDataLine = 2
        csv_varNames = {'time', 'angleFrontLeft', 'angleFrontRight', ...
                        'angleAftLeft', 'angleAftRight', ...
                        'tensionF', 'currentF', 'tensionA', 'currentA'}
    end

    properties (Constant)
        ENCODER_PRECISION = 12   % Precision of the encoder
        MAX_WING_ANGLE = 34      % Maximum wing angle, [deg]
        WING_POSITIONS = {'FrontLeft', 'FrontRight', 'AftLeft', 'AftRight'}  % Wind positions
    end

    properties (Dependent)
        shortname % Shortened version of the filename
    end

    methods

        function self = ArduData(arduinoDataFile)
            % ARDUDATA Constructor
            %   Load the arduino data file specified as input and clean it

            if nargin > 0
                self.filename = arduinoDataFile;
                self.Raw = loadcsv(self, arduinoDataFile);
            end
        end

        % ----------------------------------------------------
        % Getter/setter for dependent properties
        function shortname = get.shortname(self)
            dummy = strsplit(self.filename, filesep);
            shortname = strjoin(dummy(end - 3:end), filesep);
        end

        % ----------------------------------------------------
        % Other methods
        cleandata(self)
        calibratedTable = calibrateangles(self, uncalibratedTable)

    end
end
