function averageTable = averageangles(fullTable)
    % AVVERAGEANGLES Average the two wing angles for each set of half-wings
    %   This function averages the wing angles for each set of half-wings. It also resolves issues
    %   with non-functionning encoders (returning 0 value, having spikes, etc).
    %
    %   Note:
    %     - Integer overflow are sorted out automatically.
    %     - Encoder direction can be flipped (2022 tests have the aft encoder mounted in the
    %       opposite direction)
    %     - Encoders with noise issues or not functionning are treated as normal in this function.
    %       Special treatment will be applied with the ArduDat.averageangles function.
    % -----
    %
    % Syntax:
    %   ArduData.CALIBRATEANGLES(maxAngle) calibreate the angles according to the maximum wing angle
    %   measured on the setup.
    %
    % Inputs:
    %   uncalibratedTable : Table with uncalibrated angles.
    %   maxAngle          : Maximum positive value of the wing angle measured on the setup.
    %
    % Output:
    %   calibratedTable : Table with calibrated angles.
    %
    % See also: TunnelData, TunnelData.calibrate.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    ENCODER_PRECISION = 12;
    WING_POSITIONS = {'FrontLeft', 'FrontRight', 'AftLeft', 'AftRight'};

    nBits = 2^ENCODER_PRECISION;
    calibratedTable = uncalibratedTable;

    for iWing = 1:length(WING_POSITIONS)
        thisPos = WING_POSITIONS{iWing};
        % Unwrap integer overflow

        % Convert encoder output to degree
        angleDeg = calibratedTable.(['angle', thisPos])  * 360 / nBits;

        % Calibrate value w.r.t. maximum measured wing angle
        calibratedTable.(['angle', thisPos]) = angleDeg - max(angleDeg) + maxAngle;

    end

    % Convert time to milliseconds
    self.Cleaned.time = timetoseconds(self.Cleaned.time);

    % Remove duplicate raws
    [~, idx] = unique(self.Cleaned.time); % indexes of unique timecodes
    self.Cleaned = self.Cleaned(idx, :);

    % Calculate power and remove current and tension columns
    self.Cleaned = calculatepower(self.Cleaned, 'F');
    self.Cleaned = calculatepower(self.Cleaned, 'A');

    self.calibrateangles();

    self.Cleaned;
    disp('coucou from ArduData.cleandata');

end

function timeSec = timetoseconds(time)
    % TIMETOSECONDS Convert datetime data into seconds since start

    % Parse time properly
    time = datetime(time, 'InputFormat', 'uuuu/MM/dd''/ ''HH:mm:ss::SSS', 'TimeZone', 'UTC');

    % Converts into difference from the first time
    timeMilli = milliseconds(time - time(1)) + 1; % Shift of 1 so first time is 1 millisecond

    timeSec = timeMilli / 1000;
end

function newTable = calculatepower(table, position)
    % CALCULATEPOWER Calculate the electrical power and remove current and tension columns

    newTable = table;

    newTable.(['power', position]) = table.(['tension', position]) .* table.(['current', position]);

    newTable = removevars(newTable, ['tension', position]);
    newTable = removevars(newTable, ['current', position]);
end
