function calibratedTable = calibrateangles(self, uncalibratedTable)
    % CALIBRATEANGLES Calibrate the angles based on the raw encoder output
    %   This function unwrap angles values that were affected by interger overflow and the
    %   calibrates it so the maximum recorded position matches the maximum measured angle on the
    %   setup.
    %
    %   Note:
    %     - Integer overflow are sorted out automatically.
    %     - Encoder direction can be flipped (2022 tests have the aft encoder mounted in the
    %       opposite direction)
    %     - Encoders with noise issues or not functionning are treated as normal in this function.
    %       Special treatment will be applied with the ArduDat.averageangles function.
    % -----
    %
    % Syntax:
    %   ArduData.CALIBRATEANGLES(maxAngle) calibreate the angles according to the maximum wing angle
    %   measured on the setup.
    %
    % Inputs:
    %   uncalibratedTable : Table with uncalibrated angles.
    %   maxAngle          : Maximum positive value of the wing angle measured on the setup.
    %
    % Output:
    %   calibratedTable : Table with calibrated angles.
    %
    % See also: TunnelData, TunnelData.calibrate.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if nargin < 2
        if isempty(self.Cleaned)
            uncalibratedTable = self.Raw;
        else
            uncalibratedTable = self.Cleaned;
        end
    end

    % Defaults and constants
    nBits = 2^self.ENCODER_PRECISION;
    calibratedTable = uncalibratedTable;

    for iWing = 1:length(self.WING_POSITIONS)
        thisPos = self.WING_POSITIONS{iWing};

        % Unwrap integer overflow

        % Convert encoder output to degree
        angleDeg = calibratedTable.(['angle', thisPos])  * 360 / nBits;

        % Calibrate value w.r.t. maximum measured wing angle
        calibratedTable.(['angle', thisPos]) = angleDeg - max(angleDeg) + self.MAX_WING_ANGLE;

    end

end
