function cleandata(self)
    % CLEANDATA Clean raw Arduino data
    %   This function cleans and recalibrate the Arduino data. The following process is followed:
    %       1. Convert time to seconds
    %       2. Removal of duplicate rows
    %       3. Calculate instantaneous power from voltage and current
    %       4. Calibrate angles properly from raw encoder output
    %   The cleaned data is then saved in self.Cleaned
    %
    %   Note:
    %     The fileds with the wing angles in degree in the raw data are never used. We always
    %     re-derive from the raw encoder output.
    % -----
    %
    % Syntax:
    %   ArduData.CLEANDATA clean the raw Arduino data and save it in self.Cleaned
    %
    % Inputs:
    %   self   : An ArduData object whose Raw parameter is properly defined.
    %
    % Output:
    %   self.Cleaned: ArduData object's property updated with cleaned data
    %
    % See also: TunnelData, TunnelData.calibrate.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    KEEP_ALL_ANGLES = false; % Keep all angles in Cleaned, or just use the set average

    self.Cleaned = self.Raw;

    % Convert time to milliseconds and calculate average sampling
    self.Cleaned.time = timetoseconds(self.Cleaned.time);
    self.sampling = 1 / median(diff(self.Cleaned.time));

    % Remove duplicate raws
    [~, idx] = unique(self.Cleaned.time); % indexes of unique timecodes
    self.Cleaned = self.Cleaned(idx, :);

    % Calculate power and remove current and tension columns
    self.Cleaned = calculatepower(self.Cleaned, 'F');
    self.Cleaned = calculatepower(self.Cleaned, 'A');

    % Calibrate angles based on raw encoder value
    self.Cleaned = self.calibrateangles(self.Cleaned);

    % Take only the average for each set

    angleF = getwingsetangle(self.Cleaned.angleFrontLeft, self.Cleaned.angleFrontRight);
    angleA = getwingsetangle(self.Cleaned.angleAftLeft, self.Cleaned.angleAftRight);
    SetAngles = table(angleF, angleA);

    self.Cleaned = [self.Cleaned, SetAngles];
    self.Cleaned = movevars(self.Cleaned, "angleF", 'After', "time");
    self.Cleaned = movevars(self.Cleaned, "angleA", 'After', "angleF");

    if ~KEEP_ALL_ANGLES
        self.Cleaned = removevars(self.Cleaned, ...
                                  ["angleFrontLeft", "angleFrontRight", ...
                                   "angleAftLeft", "angleAftRight"]);
    end

end

function timeSec = timetoseconds(time)
    % TIMETOSECONDS Convert datetime data into seconds since start

    % Parse time properly
    time = datetime(time, 'InputFormat', 'uuuu/MM/dd''/ ''HH:mm:ss::SSS', 'TimeZone', 'UTC');

    % Converts into difference from the first time
    timeMilli = milliseconds(time - time(1)); % Shift of 1 so first time is 1 millisecond

    timeSec = timeMilli / 1000;
end

function newTable = calculatepower(table, position)
    % CALCULATEPOWER Calculate the electrical power and remove current and tension columns

    newTable = table;

    newTable.(['power', position]) = table.(['tension', position]) .* table.(['current', position]);

    newTable = removevars(newTable, ['tension', position]);
    newTable = removevars(newTable, ['current', position]);
end

function setAngle = getwingsetangle(left, right)
    % GETWINGSETANGLE Return the avearge angle for the wingset
    % FIXME: May need to correct for errors and imprecisions in a sensor

    % [Hacky] Disregard angles completely if the median is not roughly ~15deg
    MEDIAN_BOUNDS = [10, 20]; % [deg]

    % If wing set is not moving, return fixed value directly
    if issteady(left, right)
        setAngle = ones(size(left)) * median(left);
        return
    end

    % Either return mean of angle between left and right, or only one if sensor issue
    if ismedianok(left, MEDIAN_BOUNDS) && ismedianok(right, MEDIAN_BOUNDS)
        setAngle = (left + right) / 2;

    elseif ~ismedianok(left, MEDIAN_BOUNDS) && ismedianok(right, MEDIAN_BOUNDS)
        setAngle = right;
        warning('cleandata:getwingsetangle:diregardangledata', ...
                ['Left wing angle disrecarded due to wrong median (found %0.2f, expected '...
                 'something in [10, 20] deg.'], median(left));

    elseif ismedianok(left, MEDIAN_BOUNDS) && ~ismedianok(right, MEDIAN_BOUNDS)
        setAngle = left;
        warning('cleandata:getwingsetangle:diregardangledata', ...
                ['Right wing angle disrecarded due to wrong median (found %0.2f, expected '...
                 'something in [10, 20] deg.'], median(left));
    else
        % We assume that the set is unplugged
        setAngle = nan(size(left));

        warning('cleandata:getwingsetangle:wrongmedian', ...
                ['The angles found in the left and right wings seem to be far from the '...
                 'expected median value (15 deg).\n' ...
                 'WE ASSUME THAT THE SET IS UNPLUGGED AND RETURN A NAN ANGLE.']);
        % error('cleandata:getwingsetangle:wrongmedian', ...
        %       ['The angles found in the left and right wings seem to be far from the '...
        %       'expected median value (15 deg). Median angle calculated are %0.2f and %0.2f '...
        %       'respectively for left and right wings.'], median(left), median(right));
    end

    function bool = ismedianok(angle, okmediansbounds)
        % [Hacky] Determine if median of angles are in good range or not

        bool = median(angle) >=  okmediansbounds(1) & median(angle) <= okmediansbounds(2);
    end

    function bool = issteady(left, right)
        % ISSTEADY Determines if set is steady
        %   A normal moving wing should flap from -12 to 34 deg approximately, following a sine
        %   motion. If the standard deviation of the angle is smaller than 1 degree, we can
        %   reasonably assume that the set is steady.

        STD_STEADY = 1; % Standard deviation to be considerd steady, [deg]

        bool = std(left) < STD_STEADY && std(right) < STD_STEADY;
    end

    function bool = isunplugged(left, right)
        % ISUNPLUGGED Determines if set is unplugged
        %   A for front alone tests, the aft module was removed all together. The ardunio data kept
        %   logging noise for the angle values. This function helps detecting this scenario.
        %
        %   An unplugged system will have two medians out of bounds

        STD_STEADY = 1; % Standard deviation to be considerd steady, [deg]

        bool = std(left) < STD_STEADY && std(right) < STD_STEADY;
    end

end
