function [startIdx, startTime, steadyIdx, runTime] = findmotiontimes(self)
    % FINDMOTIONTIMES Detect the motion start and runtime trough analysis of the cleaned data.
    %   The beginning of motion is detected by finding when the angle starts
    %   deviating significantly from the mean. The criterion is either 10% of the mean or at least
    %   10x the standard deviation, whichever is the largest.
    %   The exact starting point is the last extremum before the first large deviation
    %   (calculated as the last time the derivative is 0).
    %
    % Notes:
    %   - The signal always starts stable (no motion). This is used as the baseline to dected
    %     beginning of actual movement.
    %   - This function also returns the index of the sets of wings that are steady.
    %     If a wing is steady, it is not analyzed in order to determine the start of the motion with
    %     the load sensor data.
    % -----
    %
    % Syntax:
    %   ArduData.FINDMOTIONSTART detects the index and time of the motion start.
    %
    % Inputs:
    %   self : A cleaned ArduData object.
    %
    % Output:
    %   self.Cleaned: ArduData object's property updated with cleaned data
    %
    % See also: ArduData.cleandata.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    SET_NAMES = {'angleF', 'angleA'};
    N_STABLE_VAL = 20; % Number of stable values before beginnig of motion
    DCRIT = 0.1; % Deviation from the mean to be considered a "large" deviation
    STD_NO_MOTION = 2; % A standard deviation less than 2deg means no motion

    % For each set of wings, find the start of the signal
    startIdx = [NaN, NaN];
    steadyIdx = [NaN, NaN];
    for iSet = 1:numel(SET_NAMES)
        thisSet = SET_NAMES{iSet};
        % FIXME: Is it necessary to differenciate no motion from motion case like that?
        if std(self.Cleaned.(thisSet)) > STD_NO_MOTION
            startIdx(iSet) = findsignalstart(self.Cleaned.(thisSet), N_STABLE_VAL, DCRIT);
        else
            steadyIdx(iSet) = iSet;
        end
    end

    startIdx = min(startIdx);
    startTime = self.Cleaned.time(startIdx);
    runTime = self.Cleaned.time(end) - startTime; % [s]

end

function startIdx = findsignalstart(signal, stableIdx, fixCrit)
    % FINDSIGNALSTART Find the start of the motion by looking in a specific signal.

    % Constants and defaults
    LARGE_DEV_STD = 10; % Multiplier of std to be considered 'large deviation'
    DEBUG = false;

    if nargin < 3
        fixCrit = 0;
    end

    % Find first large deviation from the mean
    meanStable = mean(signal(1:stableIdx));
    stdStable = std(signal(1:stableIdx));
    devCrit = max(fixCrit, LARGE_DEV_STD * stdStable);
    meanDev = (abs(signal - meanStable));
    largeDevIdx = find(meanDev > devCrit, 1, 'first');

    % Start point is the last extremum before first large deviation
    try
        dSignal = diff(signal(1:largeDevIdx), 1);
        % 0 first to get idx in signal and not dSignal
        inflexPts = find([0; diff(sign(dSignal))] ~= 0);
        startIdx = inflexPts(end);
    catch ME
        debugplot;
    end

    if DEBUG
        debugplot;
    end

    function debugplot
        figname = 'DEBUG-SignalStart';
        try
            close(figname);
        end
        cutoff = ones(size(signal)) * devCrit;
        figure('Name', figname);
        plot(signal, 'DisplayName', 'Signal');
        hold on;
        plot(meanDev, 'DisplayName', 'Deviation from mean');
        plot([0; dSignal], 'DisplayName', 'dSignal');
        plot(cutoff, '--k', 'DisplayName', 'Dev Crit');
        plot([largeDevIdx(1), largeDevIdx(1)], ylim, '--b', 'DisplayName', 'LargeDev Idx');
        plot([startIdx, startIdx], ylim, '--r', 'DisplayName', 'Start Idx');
        legend;
        grid on;
        xlim([0 4000]);
        pause;
    end

end
