classdef GenericData < handle
    % GENERICDATA Generic data object
    %   This class is used as a superclass to define both Arduino data and Wind Tunnel data.
    % -----
    %
    % GenericData properties:
    %
    % GenericData methods:
    %
    % GenericData constructor:
    %   Gendata = GenericData() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: ArduData, TunnelData.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties (Abstract)
        Raw table               % Raw data
        Cleaned table           % Cleaned and calibrated data
        Trimmed  table          % Trimmed and resampled data
        filename (1, :) char    % Data filename
        sampling (1, 1) double {mustBeNonnegative}  % Sampling rate, [Hz]
    end

    properties (Access = protected, Abstract)
        csv_selectedDataNames
        csv_firstDataLine
        csv_varNames
    end

    properties (Abstract, Dependent)
        shortname               % Shortened version of the filename
    end

    methods

        function self = GenericData()
            % GENERICDATA Constructor
            %   Constructor is empty on purpose

        end

        function rawTable = loadcsv(self, file)
            % LOADCSV Load the raw data from a csv file
            %   Uses class properties to define the columns to load and the variable names to give
            %   to the output

            % Tweak options before import
            opts = detectImportOptions(file);
            opts.VariableNamesLine = 1;
            if ~isempty(self.csv_selectedDataNames)
                opts.SelectedVariableNames = self.csv_selectedDataNames;
            end
            opts.DataLines = [self.csv_firstDataLine Inf];
            % If explicitely renamed, remove warning
            if ~isempty(self.csv_varNames)
                if isprop(opts, 'VariableNamingRule')
                    opts.VariableNamingRule = "preserve";
                else
                    opts.PreserveVariableNames = true;
                end
            end

            % Import and rename columns
            rawTable = readtable(file, opts);
            rawTable.Properties.VariableNames = self.csv_varNames;
        end

        function clearraw(self)
            % CLEARRAW Clear the Raw property
            self.Raw = [];
        end

        plot(self, field)  % Plot the raw data alongside the cleaned equivalent
        plotstart(self, field, startidx)  % Plot the starting point on the cleaned data
        checkstart(self, fields, startidx, newfig)  % Check the starting point for front and aft
        trimandresample(self, startIdx, runTime, newsampling) % Trim and resample cleaned data
    end

    methods (Abstract)
        cleandata(self) % Clean the raw input data
        findmotiontimes(self) % Detect the beginning of the motion
    end

end
