function trimandresample(self, startIdx, runTime, newSampling)
    % TRIMANDRESAMPLE Trim and resample data.
    %   This function removes all data before the starting index and after the runtime. It then
    %   resample everything inbetween to a new slampling rate through a spline interpolation.
    % -----
    %
    % Syntax:
    %   GenericData.trimandresample(startIdx, runTime, newSampling).
    %
    % Inputs:
    %   startIdx    : First index to keep (all data prior will be trimmed)
    %   runTime     : Total runtime in [s]
    %   newSampling : (optional) New sampling to use
    %
    % Output:
    %   self.Trimmed is updated with the trimmed data
    %
    % See also: TunnelData, ArduData.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    END_SEC_TO_TRIM = 0;  % Number of seconds to trim after the data

    if nargin < 4
        newSampling = self.sampling;
    end

    % Trim everything before the first index to keep and reset time to 0 for first data point
    trimmed = self.Cleaned(startIdx:end, :);
    trimmed.time(:) = trimmed.time(:) - trimmed.time(1);

    % Resample if necessary
    if self.sampling ~= newSampling
        time = 0:1 / newSampling:runTime;
        dummy = interp1(trimmed.time, table2array(trimmed(:, 2:end)), time);

        self.Trimmed = array2table([time', dummy]);
        self.Trimmed.Properties.VariableNames = self.Cleaned.Properties.VariableNames;
    else
        self.Trimmed = trimmed;
    end

    % Trim everything after runTime
    self.Trimmed(self.Trimmed.time >= runTime - END_SEC_TO_TRIM, :) = [];

end
