classdef Periods < handle
    % PERIODS Represent all the individual periods of a testcase
    %   This class is used to decompose the complete recorded data into individual periods for
    %   future analysis. It also implements numerous methods to resize, agglomerate and average the
    %   data over multiple periods.
    %
    % Note:
    %   Base table: {'time', ...
    %               'angleF', 'angleA', ...
    %               'powerF', 'powerA', ...
    %               'Fx_front', 'Fy_front', 'Fz_front', ...
    %               'Mx_front', 'My_front', 'Mz_front', ...
    %               'Fx_aft',  'Fy_aft', 'Fz_aft', ...
    %               'Mx_aft', 'My_aft', 'Mz_aft'}
    %
    % -----
    %
    % Periods properties:
    %
    % Periods methods:
    %
    % Periods constructor:
    %   Per = Periods() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: ArduData, TunnelData, FileLoader.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties (SetAccess = private)
        baseTable    % Base data table for period analysis
        setPosition  % Position of the set ('front' or 'aft')
        rawT         % All individual periods, [s]
        Raw          % Structure containing raw data Tables for each period
        Normalized   % Structure containing all raw data normalized on the same period
        Avg          % Structure containing the averaged data
        motion = 'flapping'  % Type of motion of the current wing set
    end

    properties (Constant, Hidden)
        PERCENTILE = [25, 75]  % Percentiles to use for average analysis
        INIT_TRIM_PC = 10      % Percentage of initial periods to discard (ramp up)
    end

    properties (Dependent)
        meanPeriod (1, 1) double % Mean period, [s]
        stdPeriod (1, 1) double  % Standard deviation of the periods, [s]
        medPeriod (1, 1) double  % Median period, [s]
        pctPeriod (1, 1) double  % 10 and 90 percentiles of the periods, [s]
        meanFreq (1, 1) double   % Mean frequency, [Hz]
        medFreq (1, 1) double    % Median frequency, [Hz]
    end

    methods

        function self = Periods(baseTable, setPosition, RefPeriods)
            % PERIODS Constructor
            %   Splits the reference signal found baseTable into individual periods

            if nargin > 0
                if nargin < 3
                    RefPeriods = [];
                end

                % --- Sore base information
                self.setPosition = validatestring(setPosition, ...
                                                  {'Front', 'Aft'}, 'Periods', 'setPosition', 2);
                self.baseTable = [baseTable(:, 1), baseTable.(self.setPosition)];

                if std(self.baseTable.angle) == 0 % Angle is not changing
                    self.motion = 'steady';
                elseif all(isnan(self.baseTable.angle))
                    self.motion = 'nowing';
                end

                % --- Split everything into periods and make base analysis
                if strcmpi(self.motion, 'flapping') || ~isempty(RefPeriods)
                    self.splitbase(RefPeriods); % Split into individual periods
                    self.normalize(); % Normalize periods using median period duration
                    self.averageall(); % Average normalized periods
                end
            end

        end

        % ----------------------------------------------------
        % Getter/setter for dependent properties

        % -- Periods
        function meanPeriod = get.meanPeriod(self)
            meanPeriod = mean(self.rawT);
        end

        function stdPeriod = get.stdPeriod(self)
            stdPeriod = std(self.rawT);
        end

        function medPeriod = get.medPeriod(self)
            medPeriod = median(self.rawT);
        end

        function pctPeriod = get.pctPeriod(self)
            pctPeriod = prctile(self.rawT, self.PERCENTILE, 3);
        end

        % -- Frequencies
        function meanFreq = get.meanFreq(self)
            meanFreq = 1 / self.meanPeriod;
        end

        function medFreq = get.medFreq(self)
            medFreq = 1 / self.medPeriod;
        end

        % ----------------------------------------------------
        % Other methods

        % Processing
        splitbase(self, RefPeriods)      % Split base table into individual periods (self.Raw)
        normalize(self)      % Normalize individual cycles on the same period
        averageall(self)        % Average normalized periods
        trim(self, nbToKeep) % Trim the number of periods by removing last ones
        rminertia(self, inertiaTable) % Remove inertia from normalized (and average) tables

        % Calculate offsets between cycles of two Period objects
        [offsetsTable] = calcphaseoffsets(self, OtherPeriods, angleType)
        % Reorganize the Period data so it starts at beginning of upstroke/downstroke
        [ReorgAvg] = reorganize(self, firstPhase)

        prune(self, pruneType)

        % Plot
        plotbase(self, dataName, newFig) % Plot the base data
        plotraw(self, dataName, newFig)  % Plot all raw data
        plotavg(self, dataName, newFig)  % Plot the averaged data

    end

    methods (Static)
        % Group and average cycle of similar phase
        [grouped] = groupandaverage(FrontPeriods, AftPeriods, offsetsTable, margin, avgType)

        function Cloned = clone(Period)
            % CLONE Clone an object (copy values rather than reference)

            Cloned = Periods;

            allProps = properties(Period);
            for iProp = 1:numel(allProps)
                thisProp = allProps{iProp};
                if ~isdependant(thisProp)
                    Cloned.(thisProp) = Period.(thisProp);
                end
            end

            function bool = isdependant(prop)
                propDetails = findprop(Periods, prop);
                bool = propDetails.Dependent;
            end

        end

    end

end
