function [offsetsTable] = calcphaseoffsets(self, OtherPeriods, angleType)
    % CALCPHASEOFFSET Calculates the phase offset between these Periods and an other ones.
    %   This function should normally be used with the Fornt wing periods as the main object and
    %   have the aft wing periods passed as the OtherPeriods. It will then calculate the phase angle
    %   between the two sets one period at a time. The phase offsets can then be used at a later
    %   time.
    %
    % Notes:
    %   - The phase angle is given in [-pi; pi] (or [-180; 180] in deg);
    %   - This analsysis uses the normalized cycles (so they all cover the same period). This
    %     facilitates the averaging of the data later on and simplifies comparisons.
    %   - Sometimes, the offset changes so much that an aft period `i` is closer to the `i+1` period
    %     of the front wing (or the inverse). When that is the case, the `i` th front period is
    %     completely disregarded. The aft period is therefore always matched with the closest front
    %     wing period.
    % -----
    %
    % Syntax:
    %   ThisPeriod.calcphaseoffset(OtherPeriod, angleType) Calculate the phase offset between
    %   ThisPeriod and the OtherPeriod. Then angleType is used to tweak the output format for the
    %   offset ('rad' or 'deg').
    %
    % Inputs:
    %   self        : a Periods object with populated self.Normalized property.
    %   OtherPeriod : a Periods object with populated self.Normalized property.
    %   angleType   : (optional) unit of the output offset angle ('rad' or 'deg', default 'rad')
    %
    % Output:
    %    phase : The phase angle between the two periods, in [-pi, pi] (or [-180, 180]).
    %
    % See also: Periods, Periods.splitbase, TestCase.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if nargin < 3
        angleType = 'rad';
    end

    offsetsTable = table('Size', [numel(self.Normalized), 3], ...
                         'VariableTypes', {'double', 'double', 'double'}, ...
                         'VariableNames', ["offset", "iSelf", "iOther"]);
    selfPeriod = self.medPeriod; % Median period

    % iPer is the normal index if everything was in sync
    % iSelf is the front index
    % iOther is the matching aft index
    iSelf = 0;
    iOther = 0;
    for iPer = 1:numel(self.Normalized)
        iSelf = iSelf + 1;
        iOther = iOther + 1;

        % Ensure we stop if we have advanced past the end of one dataset
        if iSelf < numel(self.Normalized) && iOther < numel(self.Normalized)

            % Get times of expected matching periods
            [selfTime, otherTime] = gettimes(self, OtherPeriods, iSelf, iOther);

            % Adjust matching periods if needed
            if abs(selfTime - otherTime) / (selfPeriod / 2) > 1
                if otherTime > selfTime
                    iSelf = iSelf + 1;
                else
                    iOther = iOther + 1;
                end
            end

            [selfTime, otherTime] = gettimes(self, OtherPeriods, iSelf, iOther);
            thisOffset = phasediff(selfTime, otherTime, selfPeriod);

            offsetsTable.offset(iPer) = thisOffset;
            offsetsTable.iSelf(iPer) = iSelf;
            offsetsTable.iOther(iPer) = iOther;
        else
            offsetsTable(iPer:end, :) = [];
            break
        end

    end

    if strcmpi(angleType, 'deg')
        offsetsTable.offset = rad2deg(offsetsTable.offset);
    end

end

function [selfTime, otherTime] = gettimes(self, OtherPeriods, selfIdx, otherIdx)
    % GETTIMES Returns the starting time of the 'self' and 'other' periods

    selfTime = self.Normalized(selfIdx).Data.time(1);
    otherTime = OtherPeriods.Normalized(otherIdx).Data.time(1);

end
