function [grouped] = groupandaverage(FrontPeriods, AftPeriods, offsetsTable, margin, avgType)
    % GROUPANDAVERAGE Group and average cycles of similar phase angle.
    %   This function groups the cycles of similar phase (according to some margin) and average them
    %   together for future analysis.
    %
    % Notes:
    %   -
    % -----
    %
    % Syntax:
    %   grouped = Periods.groupandaverage(FrontPeriods, AftPeriods, offsetsTable, margin, avgType)
    %   Groups the cycles w.r.t. to the offsets calculated in offsetsTable and some margin, then
    %   average them together following the avgType method (median/pctile or mean/std).
    %
    % Inputs:
    %   FrontPeriods :
    %   AftPeriods   :
    %   offsetsTable :
    %   margin       :
    %   avgType      :
    %
    % Output:
    %    phase : The phase angle between the two periods, in [-pi, pi] (or [-180, 180]).
    %
    % See also: Periods, Periods.splitbase, TestCase.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input checks
    if nargin < 4
        margin = 10;  % [deg]
    end
    if nargin < 5
        avgType = 'mean';
    end

    % --- Defaults and constants

    % --- Group offsets together and average them
    offsetVal = -180 + margin / 2:margin:180 - margin / 2;

    Avg(numel(offsetVal)) = struct;

    for iOff = 1:numel(offsetVal)
        thisOff = offsetVal(iOff);
        Avg(iOff).offset = thisOff;

        offIdx = find(abs(offsetsTable.offset - thisOff) <= margin / 2);

        if ~isempty(offIdx)
            frontIdx = offsetsTable.iFront(offIdx);
            aftIdx = offsetsTable.iAft(offIdx);

            % Average period signal (data vs time)
            [avgFront, lowFront, highFront] = averagetablestruct(FrontPeriods(frontIdx), ...
                                                                 true, 1, avgType);
            [avgAft, lowAft, highAft] = averagetablestruct(AftPeriods(aftIdx), ...
                                                           true, 1, avgType);

            % Mean value for the period
            [ovMeanFront, ovStdFront] = overallmean(FrontPeriods(frontIdx));
            [ovMeanAft, ovStdAft] = overallmean(AftPeriods(aftIdx));

            % Overall mean values for each offset period
            colNames = {'Front', 'Aft'};
            Avg(iOff).ovMean = jointables(ovMeanFront, ovMeanAft, 'time', colNames, true);
            Avg(iOff).ovStd = jointables(ovStdFront, ovStdAft, 'time', colNames, true);

            % Average cycle for each offset
            colNames = {'time', 'Front', 'Aft'};
            Avg(iOff).mean = jointables(avgFront, avgAft, 'time', colNames);
            Avg(iOff).low = jointables(lowFront, lowAft, 'time', colNames);
            Avg(iOff).high = jointables(highFront, highAft, 'time', colNames);
        else
            % This ensures we fill the table with exactly the same data format, but all data are NaN
            Avg(iOff).ovMean = nandatatable([1 8]);
            Avg(iOff).ovStd = nandatatable([1 8]);
            Avg(iOff).mean = nantimetable([500 8]);
            Avg(iOff).low = nantimetable([500 8]);
            Avg(iOff).high = nantimetable([500 8]);
        end
    end

    grouped = struct2table(Avg, 'AsArray', true);

end

function [ovMean, ovStd] = overallmean(TableStruct)
    % OVERALLMEAN Return the overall mean and std of a table structure
    % -> mean of the individual table means: ovMean = mean(mean(Table_i))

    % Take the mean values of the individual tables
    dummy = zeros(numel(TableStruct), size(TableStruct(1).Data, 2));
    for i = 1:numel(TableStruct)
        dummy(i, :) = mean(table2array(TableStruct(i).Data));
    end

    % Overall mean and std accross all tables in the structure
    ovMean = array2table(mean(dummy, 1));
    ovMean.Properties.VariableNames = TableStruct(1).Data.Properties.VariableNames;

    ovStd = array2table(std(dummy, [], 1));
    ovStd.Properties.VariableNames = TableStruct(1).Data.Properties.VariableNames;
end

function ret = nantable(tableSize, varNames)
    % Create a dummy table of NaN values with the correct variable names

    dummy(tableSize(1)) = struct;
    for i = 1:tableSize(1)
        for thisDat = varNames
            dummy(i).(thisDat{:}) = NaN;
        end
    end

    ret = struct2table(dummy);
end

function ret = nandatatable(setTableSize)
    % Create a dummy data table with two columns for front and aft wing
    DATANAMES = {'angle', 'power', 'Fx', 'Fy', 'Fz', 'Mx', 'My', 'Mz'};

    ret = table(nantable(setTableSize, DATANAMES), ...
                nantable(setTableSize, DATANAMES), 'VariableNames', {'Front', 'Aft'});
end

function ret = nantimetable(setTableSize)
    % Create a dummy datatable with time values in the first column,
    % each row corresponding to a timestep

    DATANAMES = {'angle', 'power', 'Fx', 'Fy', 'Fz', 'Mx', 'My', 'Mz'};

    dummyTable = nantable(setTableSize, DATANAMES);

    ret = table(nan(setTableSize(1), 1), dummyTable, dummyTable, ...
                'VariableNames', {'time', 'Front', 'Aft'});
end
