function normalize(self)
    % NORMALIZE Manipulates raw periods so they fill the median/mean period
    %   This populates the self.Normalized property. Normalized contains a data table (time, angles,
    %   forces, etc) for each period. The difference with self.Raw is that now the times are
    %   normalized so each cycle has the same period. Data is then interpolated so each cycle is
    %   represented by 500 elements.
    % -----
    %
    % Syntax:
    %   Periods.normalize() Manipulates the self.Raw data so they all fit the same period.
    %
    % Inputs:
    %   self : a Periods object with populated self.Raw property.
    %
    % Output:
    %    self.Normalized :
    %
    % See also: Periods, Periods.splitbase, TestCase.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Defaults and constants
    NB_PTS = 500; % Number of points in the average period;

    % --- Input checks
    % TODO: We should check if Periods.baseTable is populated

    % --- Normalizing Raw periods using median period time
    for iPer = 1:numel(self.Raw)
        % Manipulate time vector to cover exactly the median period
        ThisData = self.Raw(iPer).Data;
        initTime = ThisData.time(1);
        % Manipulate time vector to match the median period
        manipTime = ThisData.time - initTime; % Offset to 0
        manipTime = manipTime ./ manipTime(end) .* self.medPeriod; % Expend/contract to fill period
        manipTime = manipTime + initTime; % Re-offset to proper start time
        % Interpolate everything using new time vector so all have same number of points
        newTime = linspace(manipTime(1), manipTime(end), NB_PTS);

        % Interpolate everything based on new time vector
        interpData = interp1(ThisData.time, table2array(ThisData), newTime, 'spline');

        % Save normalized period
        self.Normalized(iPer).Data = array2table(interpData);
        self.Normalized(iPer).Data.Properties.VariableNames = ThisData.Properties.VariableNames;
    end

end
