function ReorgAvg = reorganize(self, firstPhase)
    % REORGANIZE Reorganize Period data so it starts at beginning of upstroke/downstroke
    %   The data in self.Avg all start when the angle reaches 0 deg during the upstroke.
    %   This function reorganize these tables so that the data is now presented from the beginning
    %   of the upstroke (or downstroke) until the end of the opposite phase instead.
    % -----
    %
    % Syntax:
    %   [AvgTable] = MyPeriod.reorganize(reorgType) Reorganize the data in MyPeriod.Avg so the begin
    %   with the `firstPhase` rather than when the angle reaches 0 during the upstroke.
    %
    % Inputs:
    %   self : a Periods object with populated self.Normalized and self.Avg properties.
    %   firstPhase : The first phase to show in the returned table ('upstroke', 'downstroke').
    %
    % Output:
    %   ReorgAvg  : A reorganized version of self.Avg
    %
    % See also: Periods.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input checks
    % TODO: We should check if self.Normalized and self.Avg are populated
    firstPhase = validatestring(firstPhase, {'upstroke', 'downstroke'}, mfilename(), 'firstPhase');

    % --- Reorganize the data

    ReorgAvg = self.Avg;
    time = self.Avg.MedianData.time;
    % Reorganize the time vector so it starts at beginning of upstroke/downstroke
    switch firstPhase
        case 'upstroke' % Upstroke start when we are at the lower flapping angle
            [~, idx] = min(self.Avg.MedianData.angle);
        case 'downstroke' % Downstroke start when we are at the higher flapping angle
            [~, idx] = max(self.Avg.MedianData.angle);
    end

    newTime = [time(idx:end); time(1:idx - 1)];
    [~, newIdx] = ismember(newTime, time);

    ReorgAvg.MedianData = self.Avg.MedianData(newIdx, :);
    ReorgAvg.LowPcData = self.Avg.LowPcData(newIdx, :);
    ReorgAvg.HighPcData = self.Avg.HighPcData(newIdx, :);

    % Overwrite the time so it is still in chronological order
    ReorgAvg.MedianData.time = time;
    ReorgAvg.LowPcData.time = time;
    ReorgAvg.HighPcData.time = time;

end
