function rminertia(self, inertiaTable)
    % RMINERTIA Remove inertia from normalized period cycles and recalculate the average period.
    % Note:
    %   The intertia measurements are only removed for the forces and moments. The time, angle and
    %   power data are not modified with this function.
    %
    % -----
    %
    % Syntax:
    %   Periods.rminertia(inertiaTable)
    %
    % Inputs:
    %   self         : a Periods object with populated self.Normalized property.
    %   inertiaTable : a table containing the average inertia measurements.
    %
    % Output:
    %    self.Normalized : Updated Normalized periods table
    %    self.Avg        : Updated Average period table
    %
    % See also: Periods, Periods.normalize, Periods.average, TestCase.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Defaults and constants
    KEEP_INDEX = 3; % 1:time, 2: angle, 3:power

    % --- Input checks
    % TODO: We should check if self.Normalized and inertiaTable are populated properly

    % Only proceed when the wing is 'flapping'. If 'steady' or 'nowing', no inertia to remove
    if ~strcmpi(self.motion, 'flapping')
        return
    end

    % --- Remove inertia from raw normalized periods
    for i = 1:numel(self.Normalized)
        aeroRes = table2array(self.Normalized(i).Data(:, KEEP_INDEX + 1:end)) - ...
           table2array(inertiaTable.MedianData(:, KEEP_INDEX + 1:end));
        self.Normalized(i).Data(:, KEEP_INDEX + 1:end)  = array2table(aeroRes);
    end

    self.averageall(); % Average normalized periods to update the self.Avg table with new values
end
