function splitbase(self, RefPeriods)
    % SPLITBASE Split base table into individual periods (self.Raw)
    %   This populates the self.Raw property. Raw contains a data table (time, angles, forces, etc)
    %   for each period.
    %
    % -----
    %
    % Syntax:
    %   Periods.splitbase() Splits the self.baseTable data into individual periods and store these
    %   in self.Raw.
    %
    % Inputs:
    %   self       : Periods object with populated baseTable property.
    %   RefPeriods : (Optional) Reference Periods object so the current Periods can follow the same
    %                decomposition (useful to split a steady signal according to a flapping one)
    %
    % Output:
    %   self.Raw : a Table with the data for each individual period. Note that the exact starting
    %   and end points of the periods are extrapolated so it corresponds exactly to an angle of
    %   0 deg.
    %
    % See also: Periods, TestCase.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input checks
    % TODO: We should check if Periods.baseTable is populated

    % --- Split everything into periods
    if isempty(RefPeriods)
        % A period starts at each negative-to-positive crossing
        % ! The index returned is the LAST NEGATIVE INDEX
        idx = find(diff(sign(self.baseTable.angle)) > 0);
        % Discard some initial periods as the movement is not well established yet
        idx = trimvect(idx, self.INIT_TRIM_PC, 'head');

        for iPer = 1:numel(idx) - 1
            % Split baseTable into individual period
            % ! Greedy: start at last neg, end in first positive
            ThisData = self.baseTable(idx(iPer):idx(iPer + 1) + 1, :);

            % Interpolate values at true start and end time of each period and overwrite table
            trueStart = interp1(ThisData.angle(1:2), table2array(ThisData(1:2, :)), 0);
            trueEnd = interp1(ThisData.angle(end - 1:end), ...
                              table2array(ThisData(end - 1:end, :)), 0);
            ThisData(1, :) = array2table(trueStart);
            ThisData(end, :) = array2table(trueEnd);
            ThisData.angle([1 end]) = [0 0]; % Force exact 0
            self.Raw(iPer).Data = ThisData;

            % Calculate true period
            self.rawT(iPer) = ThisData.time(end) - ThisData.time(1);
        end

    else
        % Split the current object (self) following the Reference Periods object
        %   -> Simply interpolate the data table on the reference time

        for iPer = 1:numel(RefPeriods.Raw)
            baseArray = table2array(self.baseTable);
            ThisData = interp1(baseArray(:, 1), baseArray, RefPeriods.Raw(iPer).Data.time);

            ThisData = array2table(ThisData);
            ThisData.Properties.VariableNames = RefPeriods.Raw(iPer).Data.Properties.VariableNames;
            self.Raw(iPer).Data = ThisData;

            % Calculate true period
            self.rawT(iPer) = ThisData.time(end) - ThisData.time(1);

        end

    end

end
