classdef ResLoader < handle
    % ResLoader Result loader
    %   This class is used to easily load the results and keep them in workspace.
    % -----
    %
    % GenericData properties:
    %
    % GenericData methods:
    %
    % GenericData constructor:
    %   None.
    %
    % Constructor inputs:
    %
    % See also: TestResult.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties (SetAccess = private)
        InertiaF        % Inertia Front Wings, N0012
        InertiaA        % Inertia Aft Wings, N0012

        Single0012      % Single Front wing, N0012 (no aft module)
        Single6409      % Single Front wing, N6409 (no aft module)

        Steady0012_3    % Tandem Steady, Dx = 3cm, N0012
        Steady0012_5    % Tandem Steady, Dx = 5cm, N0012
        Steady0012_10   % Tandem Steady, Dx = 10cm, N0012

        Tand0012_3      % Full Tandem, Dx = 3cm, N0012
        Tand0012_5      % Full Tandem, Dx = 5cm, N0012
        Tand0012_10     % Full Tandem, Dx = 10cm, N0012
        Tand6409_3      % Full Tandem, Dx = 10cm, N6409

        TandF0012_3     % Tandem Front only, Dx = 3cm, N0012
        TandF0012_5     % Tandem Front only, Dx = 3cm, N0012
        TandF0012_10    % Tandem Front only, Dx = 3cm, N0012
        TandF6409_3     % Tandem Front only, Dx = 3cm, N6409
    end

    properties (Constant, Hidden)
        % Path of the various result files
        DATA_DIR = '../data/cleaned/2023-campaign2'

        FILE_InertiaF = 'RESULTS-2023-n0012-inertiaFront_ALL.mat'
        FILE_InertiaA = 'RESULTS-2023-n0012-inertiaAft_ALL.mat'

        FILE_Single0012 = 'RESULTS-2023-n0012-frontAlone_ALL.mat'
        FILE_Single6409 = 'RESULTS-2023-n6409-frontAlone_ALL.mat'

        FILE_Steady0012_3 = 'FIXME'
        FILE_Steady0012_5 = 'FIXME'
        FILE_Steady0012_10 = 'FIXME'

        FILE_Tand0012_3 = 'RESULTS-2023-n0012-3cm-flapWindOn_ALL.mat'
        FILE_Tand0012_5 = 'RESULTS-2023-n0012-5cm-flapWindOn_ALL.mat'
        FILE_Tand0012_10 = 'RESULTS-2023-n0012-10cm-flapWindOn_ALL.mat'
        FILE_Tand6409_3 = 'RESULTS-2023-n6409-3cm-flapWindOn_ALL.mat'

        FILE_TandF0012_3 = 'RESULTS-2023-n0012-3cm-flapFront_ALL.mat'
        FILE_TandF0012_5 = 'RESULTS-2023-n0012-5cm-flapFront_ALL.mat'
        FILE_TandF0012_10 = 'RESULTS-2023-n0012-10cm-flapFront_ALL.mat'
        FILE_TandF6409_3 = 'RESULTS-2023-n6409-3cm-flapFront_ALL.mat'
    end

    methods

        function self = loadinertia(self, varargin)
            % LOADINERTIA Load inertia results
            self.InertiaF = loadresult(self.InertiaF, 'InertiaF', ...
                                       fullfile(self.DATA_DIR, self.FILE_InertiaF), ...
                                       forcereload(varargin));
            self.InertiaA = loadresult(self.InertiaA, 'InertiaA', ...
                                       fullfile(self.DATA_DIR, self.FILE_InertiaA), ...
                                       forcereload(varargin));
        end

        function self = loadsingle(self, varargin)
            % LOADSINGLE Load Single Wing set (front) results

            self.Single0012 = loadresult(self.Single0012, 'Single0012', ...
                                         fullfile(self.DATA_DIR, self.FILE_Single0012), ...
                                         forcereload(varargin));
            self.Single6409 = loadresult(self.Single6409, 'Single6409', ...
                                         fullfile(self.DATA_DIR, self.FILE_Single6409), ...
                                         forcereload(varargin));
        end

        function self = loadtandemsteady(self, varargin)
            % LOADTANDEMSTEADY Load steady tandem results

            error('Not implemented'); % FIXME

            self.Steady0012_3 = loadresult(self.Steady0012_3, 'Steady0012_3', ...
                                           fullfile(self.DATA_DIR, self.FILE_Steady0012_3), ...
                                           forcereload(varargin));
            self.Steady0012_5 = loadresult(self.Steady0012_5, 'Steady0012_5', ...
                                           fullfile(self.DATA_DIR, self.FILE_Steady0012_5), ...
                                           forcereload(varargin));
            self.Steady0012_10 = loadresult(self.Steady0012_10, 'Steady0012_10', ...
                                            fullfile(self.DATA_DIR, self.FILE_Steady0012_10), ...
                                            forcereload(varargin));

        end

        function self = loadtandemfrontflap(self, varargin)
            % LOADTANDEMFRONTFLAP Load tandem front flapping results

            self.TandF0012_3 = loadresult(self.TandF0012_3, 'TandF0012_3', ...
                                          fullfile(self.DATA_DIR, self.FILE_TandF0012_3), ...
                                          forcereload(varargin));
            self.TandF0012_5 = loadresult(self.TandF0012_5, 'TandF0012_5', ...
                                          fullfile(self.DATA_DIR, self.FILE_TandF0012_5), ...
                                          forcereload(varargin));
            self.TandF0012_10 = loadresult(self.TandF0012_10, 'TandF0012_10', ...
                                           fullfile(self.DATA_DIR, self.FILE_TandF0012_10), ...
                                           forcereload(varargin));
            self.TandF6409_3 = loadresult(self.TandF6409_3, 'TandF6409_3', ...
                                          fullfile(self.DATA_DIR, self.FILE_TandF6409_3), ...
                                          forcereload(varargin));

        end

        function self = loadtandemfull(self, varargin)
            % LOADTANDEMFULL Load tandem full flapping results
            self.Tand0012_3 = loadresult(self.Tand0012_3, 'Tand0012_3', ...
                                         fullfile(self.DATA_DIR, self.FILE_Tand0012_3), ...
                                         forcereload(varargin));
            self.Tand0012_5 = loadresult(self.Tand0012_5, 'Tand0012_5', ...
                                         fullfile(self.DATA_DIR, self.FILE_Tand0012_5), ...
                                         forcereload(varargin));
            self.Tand0012_10 = loadresult(self.Tand0012_10, 'Tand0012_10', ...
                                          fullfile(self.DATA_DIR, self.FILE_Tand0012_10), ...
                                          forcereload(varargin));
            self.Tand6409_3 = loadresult(self.Tand6409_3, 'Tand6409_3', ...
                                         fullfile(self.DATA_DIR, self.FILE_Tand6409_3), ...
                                         forcereload(varargin));
        end

        function self = loadall(self, varargin)
            % LOADALL Load all results in a single object

            self = loadinertia(self, varargin);
            self = loadsingle(self, varargin);
            self = loadtandemsteady(self, varargin);
            self = loadtandemfrontflap(self, varargin);
            self = loadtandemfull(self, varargin);
        end

    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function var = loadresult(var, varName, filename, forceReload)
    % LOADRESULT Load a result object

    if nargin < 4
        forceReload = false;
    end

    if isempty(var) || forceReload
        % Load the object in filename
        fprintf('Loading ''%s'' data... ', varName);
        var = loadobjects(filename);
        fprintf('done!\n');
    else
        warning('off', 'backtrace');
        warning('ResLoader:loadresult:alreadyloaded', ...
                ['The property ''%s'' is already loaded.\n'...
                 'Pass ''reload'' as argument in the loading function to force a reload.\n'], ...
                varName);
        warning('on', 'backtrace');
    end
end

function bool = forcereload(argList)
    % FORCERELOAD Returns true if 'reload' is in argList

    STR = 'reload';
    bool = any(strcmp(STR, argList));
end
