classdef TestCase < handle
    % TESTCASE Represent one specific test case (conditions+data)
    %   This class defines a test case by its test conditions and the resulting data.
    %   The class also implements multiple methods to clean, analyze and plot the resultign data.
    % -----
    %
    % TestCase properties:
    %
    % TestCase methods:
    %
    % TestCase constructor:
    %   Tc = TestCase() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: ArduData, TunnelData, FileLoader.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % FIXME: (M) Make it so consturctor requests file if nothing is provided
    % FIXME: (M) Fill testType property
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties
        saveFileName (1, :) char       % Name of the save file corresponding to this test
        rejected (1, 1) logical = false % If testcase should be rejected or not
    end

    properties (SetAccess = private)
        TestCond struct = struct('spacing', [0 0 0], 'airspeed', 0, 'frequency', 0, ...
                                 'temperature', 273.15, 'pressure', 101325) % Test conditions
        ArduDat (1, 1) ArduData        % Arduino Data
        TunnelDat (1, 1) TunnelData    % Wind tunnel Data (load sensors)
        Data table                     % Overall data table
        wings (:, 2) char              % Type of wings used
        testType (:, 2) char           % Type of test (front flapping, tandem flapping, etc)
    end

    properties (Dependent)
        sampling (1, 1) double {mustBeNonnegative}  % Data sampling, [Hz]
        redFreq (1, 1) double {mustBeNonnegative}   % Reduced frequency, [-]
        density (1, 1) double {mustBePositive}      % Air density, [kg/m^3]
        spacing (1, 3) double                       % Distance between the front TE aft LE, [m]
        airspeed (1, 1) double {mustBeNonnegative}  % Airspeed in the wind tunnel, [m/s]
        frequency (1, 1) double {mustBeNonnegative} % Flapping frequency, [Hz]
        temperature (1, 1) double {mustBePositive}  % Wind tunnel temperature, [K]
        pressure (1, 1) double {mustBePositive}     % Wind tunnel pressure, [Pa]
    end

    properties (Constant, Hidden)
        WING_CHORD = 0.05 % [m]
        WING_SPAN = 2 * 0.2 % [m]
    end

    methods

        function self = TestCase(spacing, airspeed, frequency, temperature, pressure, ...
                                 arduinoDataFile, tunnelDataFile)
            % TESTCASE Constructor
            %   All test condition must be specified in the constructor.
            %   The last two inputs are the name of the data files corresponding to the test case.

            if nargin > 0
                self.TestCond.spacing = spacing;
                self.TestCond.airspeed = airspeed;
                self.TestCond.frequency = frequency;
                self.TestCond.temperature = temperature + 273.15;
                self.TestCond.pressure = pressure;

                if nargin > 5
                    self.ArduDat = ArduData(arduinoDataFile);
                    self.TunnelDat = TunnelData(tunnelDataFile);
                end
            end

        end

        % ----------------------------------------------------
        % Getter/setter for dependent properties

        function sampling = get.sampling(self)
            sampling = 1 / (self.Data.time(2) - self.Data.time(1));
        end

        function redFreq = get.redFreq(self)
            redFreq = pi * self.frequency * self.WING_CHORD / self.airspeed;
        end

        function density = get.density(self)
            GAS_CST = 287.05;
            density = self.pressure / (GAS_CST * self.temperature);
        end

        % For legacy
        function spacing = get.spacing(self)
            spacing = self.TestCond.spacing;
        end

        function airspeed = get.airspeed(self)
            airspeed = self.TestCond.airspeed;
        end

        function frequency = get.frequency(self)
            frequency = self.TestCond.frequency;
        end

        function temperature = get.temperature(self)
            temperature = self.TestCond.temperature;
        end

        function pressure = get.pressure(self)
            pressure = self.TestCond.pressure;
        end

        % ----------------------------------------------------
        % Other methods
        mergedata(self)    % Merge data together
        checkstart(self, arduStartIdx, tunnelStartIdx)     % Check cleaned data sync
        mainFreq = getfreqspectrum(self, pdotFFT) % Return mean frequency from spectrum

        function clearraw(self)
            % CLEARRAW Clear the Raw properties of base data
            %   This reduces a bit the object overall size
            self.ArduDat.clearraw();
            self.TunnelDat.clearraw();
        end

        function clearall(self)
            % CLEARALL Clear complete base data sets
            %   This can be used to reduce drastically the object size once the base data are
            %   cleaned and merged.
            self.ArduDat = ArduData();
            self.TunnelDat = TunnelData();
        end

        function saveNameFromTestID(self, idTest, testNb)
            % SAVENAMEFROMTESTID Create the saveFileName from the test ID and number

            SUBDIR = '/cleaned/';

            splitIdFile = strsplit(idTest, filesep);
            saveDir = strcat(strjoin(splitIdFile(1:end - 3), filesep), ...
                             SUBDIR, splitIdFile{end - 2});
            [~, filename, ~] = fileparts(idTest);
            if ~isempty(testNb)
                self.saveFileName = fullfile(saveDir, strcat(filename, '_', num2str(testNb)));
            else
                self.saveFileName = fullfile(saveDir, filename);
            end
        end

    end

end
