function [mainFreq] = getfreqspectrum(self, plotFFT)
    % FREQDIST Return the distribution of frequencies for each set of wings.
    %   TODO: DOC
    %
    % -----
    %
    % Syntax:
    %   TestCase.freqdist(plot).
    %
    % Inputs:
    %   self     : a TestCase object
    %   poltFFT : (Optional) logical to turn-on frequency plotting
    %
    % Output:
    %    meanFreq : The mean frequency for each set of wings
    %    stdFreq  : The standard deviation of the frequency for each set of wings
    %    A plot with the distribution of frequencies (optional)
    %
    % See also: TestCase, TunnelData, ArduData.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    FIGNAME = 'TestCase - Frequencies';

    close(findobj('type', 'figure', 'name', FIGNAME));

    if nargin < 2
        plotFFT = false;
    end

    % Calculate frequency spectrum based on angle measurements
    [freqF, fftF] = calcfft(self.Data.angleF, self.sampling);
    [freqA, fftA] = calcfft(self.Data.angleA, self.sampling);

    % Main frequency
    mainFreq(1) = getmainfreq(freqF, fftF);
    mainFreq(2) = getmainfreq(freqA, fftA);

    % Plot distribution if required
    if plotFFT
        figure('Name', FIGNAME);

        % Front set
        ax1 = subplot(1, 2, 1);
        title('Front');
        hold on;
        plot(freqF, fftF);
        xline(mainFreq(1), 'r');
        xticks(0:2.5:10);

        % Aft set
        ax2 = subplot(1, 2, 2);
        title('Aft');
        hold on;
        plot(freqA, fftA);
        xline(mainFreq(2), 'r');
        xticks(0:2.5:10);

        % Axes
        linkaxes([ax1, ax2], 'xy');
        xlim([0, 10]);
        addtick(mainFreq(1), ax1);
        addtick(mainFreq(2), ax2);
        set(ax1, 'TickDir', 'out');
        set(ax2, 'TickDir', 'out');

    end

end

function [frequencies, singleSidedSpectrum] = calcfft(signal, sampling)
    % CALCFFT Return the frequency domain and the single-sided amplitude spectrum

    % Signal parameters
    if mod(length(signal), 2) ~= 0
        signal = signal(1:end - 1);
    end
    nts = length(signal);

    % Fourier transform
    transform = fft(signal);
    fT2 = abs(transform / nts); % Two-sided spectrum

    singleSidedSpectrum = fT2(1:nts / 2 + 1); % Single-sided spectrum
    singleSidedSpectrum(2:end - 1) = 2 * singleSidedSpectrum(2:end - 1);
    frequencies = sampling * (0:(nts / 2)) / nts; % Frequency domain
end

function mainFreq = getmainfreq(frequencies, fftSpectrum)
    % GETMAINFREQ Return the main frequency from the fft spectrum

    % FIXME: ISSUE HERE?

    [~, mainFeqIdx] = max(fftSpectrum(2:end)); % Make sure it is not zero
    mainFreq = frequencies(1 + mainFeqIdx);
end
