function mergedata(self)
    % MERGEDATA Synchronize and merge the datasets from wind tunnel and arduino.
    %   The acquisition of data is realized from two different sources on two different machines:
    %       - load sensors (TunnelData)
    %       - wing position (ArduData)
    %   This function synchronizes both data sources by detecting the beginning of the movement on
    %   both data sets. It then interpolates the data over the time domain in order to have the same
    %   time code for each set. After synchronization, the two datasets are stored into a single
    %   table (TestCase.Data) for easier analysis.
    %
    % Note:
    %   We must determine if a wing is steady using the Arduino data in order to discard it when
    %   searching for the start in the load sensor.
    % -----
    %
    % Syntax:
    %   TestCase.MERGEDATA() synchronize and merge the cleaned TunnelData and ArduData already
    %   linked in the TestCase object, and store it in TestCase.Data
    %
    % Inputs:
    %   self : a TestCase object
    %
    % Output:
    %   upated value for TestCase.Data
    %
    % See also: TestCase, TunnelData, ArduData.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    DEBUG = false;
    VARNAMES = {'angle', 'power', 'Fx', 'Fy', 'Fz', 'Mx', 'My', 'Mz'};

    if DEBUG
        fprintf(2, 'ArduFile: %s\nTunnelFile: %s\n\n', ...
                self.ArduDat.shortname, self.TunnelDat.shortname);
    end

    % Detect if TunnelData and ArduData are already cleaned
    inputvalidator(self.TunnelDat, 'TestCase.TunnelDat cannot be empty.');
    inputvalidator(self.TunnelDat.Cleaned, 'TestCase.TunnelDat must be cleaned.');
    if ~isempty(self.ArduDat) % Can be empty for static tests
        inputvalidator(self.ArduDat.Cleaned, 'TestCase.ArduDat must be cleaned.');
    end

    % Detect first motion in Arduino and Tunnel data sets
    [arduStartIdx, arduStartTime, steadyIdx, arduRunTime] = self.ArduDat.findmotiontimes();
    [tunnelStartIdx, tunnelStartTime, tunnelRunTime] = self.TunnelDat.findmotiontimes(steadyIdx);

    % Trim the datasets accordingly and resample them based on the same time vector
    maxSampling = max(self.ArduDat.sampling, self.TunnelDat.sampling);
    minRunTime = min(arduRunTime, tunnelRunTime);

    if DEBUG
        fprintf(2, ['ArduStart - Idx: %.2f, startTime %.2f s, steadyIdx: [%d, %d], '...
                    'runTime %.2f s \n'], arduStartIdx, arduStartTime, steadyIdx, arduRunTime);
        fprintf(2, 'TunnelStart - Idx: %.2f, startTime %.2f s, runTime %.2f s \n', ...
                tunnelStartIdx, tunnelStartTime, tunnelRunTime);
        fprintf(2, 'Sampling: %.2f Hz, runTime: %.2f s\n', maxSampling, minRunTime);

        % Plots
        self.checkstart(arduStartIdx, tunnelStartIdx);
        pause;
    end

    self.ArduDat.trimandresample(arduStartIdx, minRunTime, maxSampling);
    self.TunnelDat.trimandresample(tunnelStartIdx, minRunTime, maxSampling);

    % Concatenate everything into a single table
    self.Data.time = self.ArduDat.Trimmed.time;
    self.Data.Front = [self.ArduDat.Trimmed(:, {'angleF', 'powerF'}), ...
                       self.TunnelDat.Trimmed(:, {'Fx_front', 'Fy_front', 'Fz_front', ...
                                                  'Mx_front', 'My_front', 'Mz_front'})];
    self.Data.Aft = [self.ArduDat.Trimmed(:, {'angleA', 'powerA'}), ...
                     self.TunnelDat.Trimmed(:, {'Fx_aft', 'Fy_aft', 'Fz_aft', ...
                                                'Mx_aft', 'My_aft', 'Mz_aft'})];
    self.Data.Front.Properties.VariableNames = VARNAMES;
    self.Data.Aft.Properties.VariableNames = VARNAMES;

end

% --------------------------------------------------------------------------------------------------
function inputvalidator(Table, errMsg)
    % INPUTVALIDATOR Return error if input table is empty

    if isempty(Table)
        error('TestCase:emptyData', errMsg);
    end

end
