classdef TestResult < handle
    % TESTRESULT Processed TestCase
    %   TODO.
    %
    % -----
    %
    % TestResult properties:
    %
    % TestResult methods:
    %
    % TestResult constructor:
    %   Tc = TestResult() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: TestCase.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % FIXME: (M) Make it so consturctor requests file if nothing is provided
    % ----------------------------------------------------------------------------------------------
    % NOTES:
    %   Old system was to have ExpResults(iTest) for each file
    %   Then ResData(iFile, iTest)
    %
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties (SetAccess = private)
        TestCond
        PeriodsF
        PeriodsA
        offsetsTable
        OffsetGroups
        WING_CHORD
    end

    properties
        rejected
        inertia_removed = false
        inertia_idx = []
    end

    properties (Constant, Hidden)
        OFFSET_STEPS = 10
        OFFSET_AVG = 'mean'
    end

    properties (Dependent)
        Data          % Original TestResult Data
        trueFreq      % True measured frequency, based on period analysis
        meanFreq      % Mean true frequency
        meanRedFreq   % Mean true frequency
    end

    properties (SetAccess = private, Hidden)
        Data_   % Private version of Data
    end

    properties (Dependent)
        redFreq (1, 1) double {mustBeNonnegative}   % Reduced frequency, [-]
        density (1, 1) double {mustBePositive}      % Air density, [kg/m^3]
    end

    methods

        function self = TestResult(TestCase)
            % TESTRESULT Constructor
            %   Automatically process the TestCase by splitting it by phase for analysis.

            if nargin > 0
                % Pass useful values from TestCase to the TestResult
                self.rejected = TestCase.rejected;
                self.Data = TestCase.Data;
                self.TestCond = TestCase.TestCond;
                self.WING_CHORD = TestCase.WING_CHORD;

            end

        end

        % ----------------------------------------------------
        % Getter/setter for dependent properties
        function set.Data(self, val)
            self.Data_ = val;
        end

        function Data = get.Data(self)
            Data = [];

            if ~isempty(self.Data_)
                Data = self.Data_;
            elseif ~isempty(self.PeriodsF.baseTable) && ~isempty(self.PeriodsA.baseTable)
                Data = table(self.PeriodsA.baseTable.time, ...
                             self.PeriodsF.baseTable(:, 2:end), ...
                             self.PeriodsA.baseTable(:, 2:end));
                Data.Properties.VariableNames = {'time', 'Front', 'Aft'};
            end
        end

        function redFreq = get.redFreq(self)
            redFreq = pi * self.TestCond.frequency * self.WING_CHORD / self.TestCond.airspeed;
        end

        function density = get.density(self)
            GAS_CST = 287.05;
            density = self.TestCond.pressure / (GAS_CST * self.TestCond.temperature);
        end

        function trueFreq = get.trueFreq(self)
            trueFreq = [self.PeriodsF.medFreq, self.PeriodsA.medFreq];
        end

        function meanFreq = get.meanFreq(self)
            meanFreq = mean(self.trueFreq);
        end

        function meanRedFreq = get.meanRedFreq(self)
            meanRedFreq = pi * self.meanFreq * self.WING_CHORD / self.TestCond.airspeed;
        end

        % ----------------------------------------------------
        % Other methods

        function splitperiods(self)
            % SPLITPERIODS Split signal into individual periods

            % Split into periods
            self.PeriodsF = Periods(self.Data, 'Front');
            self.PeriodsA = Periods(self.Data, 'Aft');

            % If one set is steady, use the flapping set as basis for period decomposition
            if strcmpi(self.PeriodsF.motion, 'steady')
                self.PeriodsF = Periods(self.Data, 'Front', self.PeriodsA);
            elseif strcmpi(self.PeriodsA.motion, 'steady')
                self.PeriodsA = Periods(self.Data, 'Aft', self.PeriodsF);
            end

            % Trim objects so they have the same number of cycles
            if ~strcmpi(self.PeriodsF.motion, 'nowing') && ~strcmpi(self.PeriodsA.motion, 'nowing')
                nPeriods = min([numel(self.PeriodsF.Raw), numel(self.PeriodsA.Raw)]);
                self.PeriodsF.trim(nPeriods);
                self.PeriodsA.trim(nPeriods);
            end

        end

        function getoffsets(self)
            % GETOFFSETS Calculate offsets between the two sets

            self.offsetsTable = self.PeriodsF.calcphaseoffsets(self.PeriodsA, 'deg');
            self.offsetsTable.Properties.VariableNames = {'offset', 'iFront', 'iAft'};
        end

        function groupoffsets(self)
            % GROUPOFFSETS Group and average cycles of same offset

            self.OffsetGroups = Periods.groupandaverage(self.PeriodsF.Normalized, ...
                                                        self.PeriodsA.Normalized, ...
                                                        self.offsetsTable, ...
                                                        self.OFFSET_STEPS, self.OFFSET_AVG);
        end

        function rminertia(self, ResInertia)
            % RMINERTIA Remove interia from the normalized periods results

            self.PeriodsF.rminertia(ResInertia.PeriodsF.Avg);
            self.PeriodsA.rminertia(ResInertia.PeriodsA.Avg);

            self.inertia_removed = true;

        end

        function str = strcond(self, strFormat, freqFormat)
            % STRCOND Format test condition into string

            % --- Input check
            if nargin < 2
                strFormat = 'full';
            end
            if nargin < 3
                freqFormat = 'true';
            end

            % --- Format string

            switch freqFormat
                case 'ideal'
                    strFreq = sprintf('f = %0.2f Hz (k = %0.2f)', ...
                                      self.TestCond.frequency, self.redFreq);
                case 'true'
                    strFreq = sprintf('f = [%0.2f, %0.2f] Hz (k ~ %0.2f)', ...
                                      self.trueFreq, self.meanRedFreq);
                case 'meanfreq'
                    strFreq = sprintf('f = %0.2f Hz (k = %0.2f)', self.meanFreq, self.meanRedFreq);
                otherwise % Same as 'true'
                    strFreq = sprintf('f = [%0.2f, %0.2f] Hz (k ~ %0.2f)', ...
                                      self.trueFreq, self.meanRedFreq);
            end

            % Different string formats
            switch strFormat
                case 'short'
                    str =  sprintf('Dx = %d cm | V = %0.1f m/s | %s', ...
                                   self.TestCond.spacing(1) * 100, self.TestCond.airspeed, ...
                                   strFreq);
                case 'title'
                    % Get proper median frequency for each set
                    str = sprintf('Dx = %d cm | V = %0.1f m/s | %s', ...
                                  self.TestCond.spacing(1) * 100, self.TestCond.airspeed, strFreq);
                otherwise
            end

        end

        plotoffsets(self, dataNames, plotType, newFig)
        plotraw(self, dataName, figTitle)
        plotavg(self, dataNames, figTitle)
        prune(self, pruneType)

        % coeff

    end

    methods (Static)

        function TestResCopy = prunedcopy(TestRes, pruneType)
            % PRUNEDCOPY Makes a copy of TestRes object and prune it directly
            %   FIXME : Periods just copied handle, not true value copy

            DO_NOT_CLONE = {'trueFreq', 'meanFreq', 'meanRedFreq', 'redFreq', 'density'};

            TestResCopy = TestResult;

            allProps = properties(TestRes);
            for iProp = 1:numel(allProps)
                thisProp = allProps{iProp};

                if ~any(strcmpi(thisProp, DO_NOT_CLONE))
                    TestResCopy.(thisProp) = TestRes.(thisProp);
                end
            end

            % Properly clone the Period objects value rather than handle
            TestResCopy.PeriodsF = Periods.clone(TestRes.PeriodsF);
            TestResCopy.PeriodsA = Periods.clone(TestRes.PeriodsA);

            % Prune
            TestResCopy.prune(pruneType);

        end

    end
end
