function self = prune(self, pruneType)
    % PRUNE Prune some redundant data to save disk space and improve loading time
    %   The initial data is duplicated a lot. This can duplication can be improved by pruning the
    %   redundant data everywhere possible. The pruned data can then be recreated/accessed later on
    %   from the detailed data if needed.
    %
    % -----
    %
    % Syntax:
    %   TestResult.prune(pruneType) Prune the TestResult object (and eventually Periods object
    %   within it) according to the pruneType used. The pruneType can be an array of different
    %   pruning strategies.
    %
    %
    % Inputs:
    %   self      : a TestResult object. Ideally fully formed and calculated so the pruned data can
    %               be (partially) reconstructed from remaining data.
    %   pruneType : Type of pruning to use. Can be a character vector or a list of pruning types.
    %               The following options are implemented:
    %                   - baseData     : clear TestResult.Data
    %                   - offset       : clear TestResult.offsetsTable and TestResult.OffsetGroups
    %                   - periods_base : clear TestResult.PeriodsA/F.baseData
    %                   - periods_raw  : clear TestResult.PeriodsA/F.Raw
    %                   - periods_norm : clear TestResult.PeriodsA/F.Normalized
    %                   - periods_avg  : clear TestResult.PeriodsA/F.Avg
    %                   - periods_all  : clear TestResult.PeriodsA/F completely
    %
    % Output:
    %   self : The original TestCase object, with cleared fields.
    %
    %
    % See also: TestResults, Periods.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input validation
    pruneType = string(pruneType);

    % --- Pruning
    for iPrune = pruneType
        if any(strcmpi(iPrune, {'baseData', 'base'}))
            self.Data_ = [];
        elseif any(strcmpi(iPrune, {'offset', 'offsets'}))
            self.offsetsTable = [];
            self.OffsetGroups = [];
        elseif strcmpi(iPrune, 'periods_base')
            self.PeriodsF.prune('baseData');
            self.PeriodsA.prune('baseData');
        elseif strcmpi(iPrune, 'periods_raw')
            self.PeriodsF.prune('raw');
            self.PeriodsA.prune('raw');
        elseif strcmpi(iPrune, 'periods_norm')
            self.PeriodsF.prune('normalized');
            self.PeriodsA.prune('normalized');
        elseif strcmpi(iPrune, 'periods_avg')
            self.PeriodsF.prune('average');
            self.PeriodsF.prune('average');
        elseif strcmpi(iPrune, 'periods_all')
            self.PeriodsF = Periods;
            self.PeriodsA = Periods;
        end
    end

end
