classdef TunnelData < GenericData
    % TUNNELDATA Represent a single Wind Tunnel Data file
    %   This class loads, clean and store the Wind Tunnel Data corresponding to a single test case.
    % -----
    % TunnelData properties:
    %
    % TunnelData methods:
    %
    % TunnelData constructor:
    %   Tunnel = TunnelData() creates an empty object.
    %
    % Constructor inputs:
    %
    % See also: GenericData, TestCase.
    %
    % <a href="https://gitlab.uliege.be/mecharaptor/data-processing">Repository (online)</a>

    % ----------------------------------------------------------------------------------------------
    % FIXME: (L) Make it so consturctor requests file if nothing is provided
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    properties
        Raw = []    % Raw data
        Cleaned     % Cleaned and calibrated data
        Trimmed     % Trimmed and resampled data
        filename    % Data filename
        sampling = 1e3  % Sampling rate, [Hz]
    end

    properties (Access  = protected)
        csv_selectedDataNames = {}
        csv_firstDataLine = 1
        csv_varNames = {'Fx_front', 'Fy_front', 'Fz_front', ...
                        'Mx_front', 'My_front', 'Mz_front', ...
                        'Fx_aft', 'Fy_aft', 'Fz_aft', ...
                        'Mx_aft', 'My_aft', 'Mz_aft'}
    end

    properties (Dependent)
        shortname % Shortened version of the filename
    end

    methods

        function self = TunnelData(tunnelDataFile)
            % TUNNELDATA Constructor
            %   Load the wind tunnel data file specified as input and clean it

            if nargin > 0
                self.filename = tunnelDataFile;
                self.Raw = loadcsv(self, tunnelDataFile);
            end
        end

        % ----------------------------------------------------
        % Getter/setter for dependent properties
        function shortname = get.shortname(self)
            dummy = strsplit(self.filename, filesep);
            shortname = strjoin(dummy(end - 4:end), filesep);
        end

        % ----------------------------------------------------
        % Other methods
        function self = updatesampling(self, val)

            if nargin < 2
                val = self.sampling;
            end
            oldSampling = self.sampling;

            if oldSampling ~= val && ...
                    any(ismember(self.Raw.Properties.VariableNames, 'time')) && ...
                    ~isempty(self.Cleaned)

                warning('TunnelData:updatesampling:samplingAlreadySet', ...
                        ['The sampling was already set by the user to %.2f. \n'...
                         'Changing it to %.2f will modify TunnelData.Raw.time.\n'...
                         'The cleaned data sampling WILL NOT MATCH ANYMORE. Please redo \n '...
                         'any processing already done.\n'], oldSampling, val);
            end

            self.sampling = val;
            ts = 1 / val;
            time = (0:ts:(size(self.Raw(:, 1)) - 1) * ts)';

            if any(ismember(self.Raw.Properties.VariableNames, 'time'))
                self.Raw.time = time;
            else
                self.Raw = [table(time), self.Raw];
            end

        end

        % ----------------------------------------------------
        % Other methods
        cleandata(self, cutoff)
        calibreatedTable = calibrateparasiticforce(self, uncalibratedTable, forceName, momentName)

    end
end
