function calibratedTable = calibrateparasiticforce(self, uncalibratedTable, forceName, momentName)
    % CALIBRATEPARASITICFORCE Calibrate forces by removing parastic contributions
    %   The ATI Gamma load cells record a parasitic Fx (resp Fz) force when a My (resp Mx) moment is
    %   applied to it.
    %   Thankfully, this effect evolves linearily and can therefore be easily accounted for.
    %   This scripts uses calibration runs to find the slope of the F-M curve. It then subtracts
    %   the parastic force based on the measured moment for each time step.
    %
    % Process:
    %   This calibration is done by looking at various measurements conducted under no-wind
    %   conditions.
    %   A mass was moved along a pole to apply a small moment along the Y (resp X) direction. One
    %   data file was created per position of the mass (i.e., per moment).
    %   This method works by taking the average value of the force and moment for each recording.
    %   Then the slope of the curve formed by these mean values is extracted. This slope is finally
    %   used to determine the value of the parasitic force for each value of the moment recoreded in
    %   the test data. The parastic force is the subtracted and the calibrated table is returned.
    % -----
    %
    % Syntax:
    %   TunnelData.CALIBRATEPARASITICFORCE(uncalibratedTable, forceName, momentName) calibrate the
    %   system by removing the parasitic Force 'forceName' originating from the moment `momentName`.
    %
    % Inputs:
    %   uncalibratedTable : Un-calibrated data table
    %   forceName         : Force to calibrate ('Fx', 'Fy' or 'Fz')
    %   momentName        : Moment at the origin of the parasitic force ('Mx', 'My' or 'Mz')
    %
    % Output:
    %   calibratedTable: The data table after calibration
    %
    % See also: TunnelData, TunnelData.cleandata.
    %
    % ----------------------------------------------------------------------------------------------
    % FIXME: (L) Use Matlab lowpass function directly to filter?
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults arguments and input checks
    arguments
        self TunnelData
        uncalibratedTable table
        forceName (1, :) char = 'Fx'
        momentName (1, :) char = 'My'
    end

    % Abbreviation
    fm = [forceName, momentName];

    % Calibration factor for Fx-My and Fz-Mx
    persistent kFront kAft  % Persistent to avoid reloading them multiple times

    if isempty(kFront) || isempty(kAft)
        kFront = struct('FxMy', [], 'FzMx', [], 'FyMz', []);
        kAft = struct('FxMy', [], 'FzMx', [], 'FyMz', []);
    end

    % Get the calibration slope
    if isempty(kFront.(fm)) || isempty(kAft.(fm))

        % Tunnel Data Base directory always 2 or 3 dir above the file
        % ex: campaign/windtunnelData/year/wingtype/distance/flapType/file.txt
        % -> base directory = campaign/windtunnelData/year/
        pathParts = strsplit(self.filename, filesep);
        if any(strcmpi(pathParts, 'inertia')) || any(strcmpi(pathParts, 'frontAlone'))
            parentDirIdx = 3;
        else
            parentDirIdx = 4;
        end
        tunnelDataBaseDir = fullfile(filesep, pathParts{1:end - parentDirIdx}, filesep);

        % Calibration slope from files
        kFront.(fm) = findcalibslope(tunnelDataBaseDir, forceName, momentName, 'Front');
        kAft.(fm) = findcalibslope(tunnelDataBaseDir, forceName, momentName, 'Aft');
    end

    % Apply calibration by removing parasitc force
    calibratedTable = applycalibration(uncalibratedTable, kFront.(fm), forceName, momentName, ...
                                       'front');
    calibratedTable = applycalibration(calibratedTable, kAft.(fm), forceName, momentName, 'aft');

end

function slope = findcalibslope(tunnelDataDir, forceName, momentName, position)
    % FINDCALIBSLOPE Find linear coefficient for the calibration of parasitic effects

    % Get calibration files name
    calibFiles = fullfile(tunnelDataDir, 'calibration', ...
                          ['calib', position, forceName, momentName, '_*']);

    % Get all relevant calibration files
    allFiles = dir(calibFiles);
    if isempty(allFiles)
        warning('off', 'backtrace');
        warning('TunnelData:calibrateparasiticforce:noFileFound', ...
                ['Impossible to find the file \n'...
                 '%s \n'...
                 'for the calibration of the parastic force %s '...
                 'created by the moment %s for the %s wing.\n'...
                 'Continuing without this calibration...\n'], ...
                calibFiles, forceName, momentName, position);
        warning('on', 'backtrace');
        slope = 0;
        return
    end

    % Find the value of the force and moment for each calibration file
    meanF = zeros(size(allFiles));
    meanM = zeros(size(allFiles));
    for iFile = 1:numel(allFiles)
        file = allFiles(iFile);

        % Determine columns id to look for
        if contains(file.name, 'calibFront')
            set = 'front';
        elseif contains(file.name, 'calibAft')
            set = 'aft';
        end

        % Load calibration data and get mean force and moment
        CalibData = TunnelData(fullfile(file.folder, file.name));
        meanF(iFile) = mean(CalibData.Raw.([forceName, '_', set]));
        meanM(iFile) = mean(CalibData.Raw.([momentName, '_', set]));

    end

    slope = mean(diff(meanF) ./ diff(meanM));
end

function calibTable = applycalibration(uncalibratedTable, slope, ...
                                       forceName, momentName, position)
    % APPLYCALIBRATION Apply the calibration to the uncalibrated table

    calibTable = uncalibratedTable;
    calibTable.([forceName, '_', position]) = uncalibratedTable.([forceName, '_', position]) - ...
                                              slope .* ...
                                              uncalibratedTable.([momentName, '_', position]);

end
