function cleandata(self, cutoff)
    % CLEANDATA Clean raw wind tunnel data
    %   This function applies a low-pass filter to remove noise from wind tunnel raw data. It then
    %   calls the calibration method to remove the parasitic force measured by the load cell when a
    %   moment is applied.
    %   Cleaned data are then saved in self.Cleaned.
    % -----
    %
    % Syntax:
    %   TunnelData.CLEANDATA clean the raw wind tunnel data and save it in self.Cleaned
    %
    % Inputs:
    %   self   : A TunnelData object whose Raw parameter is properly defined.
    %   cutoff : Cutoff frequency for the low-pass filter
    %
    % Output:
    %   self.Cleaned: TunnelData object's property updated with cleaned data
    %
    % See also: TunnelData, TunnelData.calibrate.
    %
    % ----------------------------------------------------------------------------------------------
    % FIXME: (L) Use Matlab lowpass function directly to filter?
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if nargin < 2
        cutoff = 0;
    end

    % If sampling has not been explicitely set, make sure the default one is set in Raw
    self.updatesampling();

    % Low-pass filtering (on data columns only, not time)
    if cutoff ~= 0
        filtered = lpfilter(self.Raw(:, 2:end), cutoff, self.sampling);
    else
        filtered = self.Raw(:, 2:end); % No noise to filter if nothing moves (requested_freq = 0)
    end
    self.Cleaned = [self.Raw(:, 1), filtered];

    % Calibration of parasitic forces
    self.Cleaned = self.calibrateparasiticforce(self.Cleaned, 'Fx', 'My');
    self.Cleaned = self.calibrateparasiticforce(self.Cleaned, 'Fz', 'Mx');

end

function filteredTable = lpfilter(rawTable, cutoff, sampling)
    % LPFILTER Apply a low-pass filter to the raw data table
    %   The `raw` input and `filtered` output are both tables

    % Design a butterworth IIR filter
    nyq = sampling / 2;
    [bb, aa] = butter(3, cutoff / nyq, 'low');

    % Apply filter
    filteredTable = array2table(filtfilt(bb, aa, table2array(rawTable)));
    % Re-apply variable names
    filteredTable.Properties.VariableNames = rawTable.Properties.VariableNames;
end
