function [startIdx, startTime, runTime] = findmotiontimes(self, steadyIdx)
    % FINDMOTIONSTART Detect the motion start and run time trough analysis of the cleaned data.
    %   The beginning of motion is detected by finding when the Z force starts
    %   deviating significantly from the mean. The criterion is either 10% of the mean or at least
    %   10x the standard deviation, whichever is the largest.
    %   The exact starting point is the last MINIMUM in the signal before the first large
    %   deviation, not just any extremum.
    %
    % Note:
    %   The signal always starts stable (no motion). This is used as the baseline to dected
    %   beginning of actual movement.
    % -----
    %
    % Syntax:
    %   TunnelData.FINDMOTIONSTART detects the index and time of the motion start.
    %
    % Inputs:
    %   self : A cleaned TunnelData object.
    %
    % Output:
    %   self.Cleaned: TunnelData object's property updated with cleaned data
    %
    % See also: TunnelData.cleandata.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    SET_NAMES = {'front', 'aft'};
    DATA_TO_USE = 'Mx'; % Data to use as basis for signal start determination

    DCRIT.Fx = 0.1; % Deviation from the mean to be considered a "large" deviation (in absolute)
    DCRIT.Fz = 0.1; % Deviation from the mean to be considered a "large" deviation (in absolute)
    DCRIT.Mx = 0.005; % Deviation from the mean to be considered a "large" deviation (in absolute)
    DCRIT.My = 0.005; % Deviation from the mean to be considered a "large" deviation (in absolute)

    TRIM_MS = 10; % Number of milliseconds to trim initially to avoid unnecessary noise

    % Remove elecromagnetic noise coming from recording start
    trimmed = self.Cleaned(TRIM_MS + 1:end, :);

    % For each set of wings, find the start of the signal
    startIdx = [NaN, NaN];

    for iSet = 1:numel(SET_NAMES)
        thisDataPt = sprintf('%s_%s', DATA_TO_USE, SET_NAMES{iSet});
        if steadyIdx(iSet) ~= iSet
            startIdx(iSet) = findsignalstart(trimmed, DATA_TO_USE, SET_NAMES{iSet}, self.sampling / 5, DCRIT.(DATA_TO_USE), ...
                                             self.filename);
        end
    end
    startIdx = TRIM_MS + min(startIdx);
    startTime = self.Cleaned.time(startIdx);
    runTime = self.Cleaned.time(end) - startTime; % [s]
end

function startIdx = findsignalstart(trimmed, dataToUse, setName, stableIdx, fixCrit, filename)
    % FINDSIGNALSTART Find the start of the motion by looking in a specific signal.

    % Constants and defaults
    LARGE_DEV_STD = 10; % Multiplier of std to be considered 'large deviation'
    SAMPLING = 1000;
    DEBUG = false;

    if nargin < 5
        fixCrit = 0;
    end

    thisData = sprintf('%s_%s', dataToUse, setName);
    signal = trimmed.(thisData);
    startIdx = [];
    try
        % Find first large deviation from the mean
        meanStable = mean(signal(1:stableIdx));
        stdStable = std(signal(1:stableIdx));
        devCrit = max(fixCrit, LARGE_DEV_STD * stdStable);
        meanDev = (abs(signal - meanStable));
        largeDevIdx = find(meanDev > devCrit, 1, 'first');

        % Start point is the last extremum before first large deviation
        dSignal = diff(signal(1:largeDevIdx), 1) / (1 / SAMPLING);
        extremums = find([0; diff(sign(dSignal))] ~= 0); % 0 first to get idx in signal not dSignal
        startIdx = extremums(end);

        % TEST - LAST INFLEXION POINT
        ddSignal = diff(dSignal(1:end), 1) / (1 / SAMPLING);
        inflPts = find([0; diff(sign(ddSignal))] ~= 0); % 0 first to get idx in signal not dSignal
        startIdx = extremums(end - 1);

        % Starting point must be a MINIMUM, not any extremum (FIXME: triple check)
        %         [~, newIdx] = min(meanDev(startIdx:largeDevIdx(1)));
        %         startIdx = (startIdx - 1) + newIdx;
    catch EX
        fprintf(2, 'ISSUE WITH FILE\n %s\n', filename);
        debugfig();
        rethrow(EX);
    end

    if DEBUG
        debugfig();
    end

    function debugfig
        close all;

        figure('Name', 'DEBUG - TunnelDat FindMotionTimes');
        ax1 = subplot(2, 2, 1); % FZ (Top Left)
        debugplot('Fz', dataToUse);

        ax2 = subplot(2, 2, 2); % FX (Top Right)
        debugplot('Fx', dataToUse);

        ax3 = subplot(2, 2, 3); % My (Bottom Left)
        debugplot('My', dataToUse);

        ax4 = subplot(2, 2, 4); % Mx (Bottom Left)
        debugplot('Mx', dataToUse);

        linkaxes([ax1, ax2, ax3, ax4], 'x');
        linkaxes([ax1, ax2], 'y');
        linkaxes([ax3, ax4], 'y');

        pause;
    end

    function debugplot(force, usedForce)

        plot(trimmed.(sprintf('%s_%s', force, setName)), 'DisplayName', force);
        hold on;
        if strcmpi(force, usedForce)
            plot(meanDev, 'DisplayName', 'Deviation from mean');
            plot([0; dSignal], 'DisplayName', 'dSignal');
            plot([0; ddSignal / 10], 'DisplayName', 'ddSignal/10');

            yline(devCrit, '--k', 'DisplayName', 'Dev Crit');
            xline(largeDevIdx(1), '--b', 'DisplayName', 'LargeDev Idx');
        end

        if ~isempty(startIdx)
            xline(startIdx, '--r', 'DisplayName', 'Start Idx');
        end
        hold off;

        if strcmpi(force, usedForce)
            legend('Location', 'NorthWest');
        end

        title(force);
        xlim([0 8000]);
        xlabel('time [s]');
        grid on;

    end

end
