% FIG_INTRO Create figures used in the introduction of the flapping chapter
%   This script load the appropriate data and generate the various figures used in my thesis.
%   The figures are automatically saved using matlab2tikz.
%
%   The title and some annotations are then added afterwards for easier analysis when generated with
%   Matlab, they may not all be present in the exported figure for clarity in the dissertation.
% -----
%
% Syntax:
%   FIG_INTRO;
%
% Inputs:
%   None
% Output:
%   The following figures are created and saved:
%       - Frequency difference between front and aft sets for full tandem flapping
%       - Frequency distribution for front and aft sets during a single test
%       - Illustrate how inertia is removed from raw data to form only aerodynamic contribution
%
% See also: TestCase, TestResult.

% ----------------------------------------------------------------------------------------------
% matlab2tikz: https://github.com/matlab2tikz/matlab2tikz
% ----------------------------------------------------------------------------------------------
% (c) Copyright 2022-2024 University of Liege
% Author: Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% MIT License
% Repo: https://gitlab.uliege.be/mecharaptor/data-processing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% --- Initial setup
fig_setup;

% --- Load data required for this file
printsection('0. Data loading');

RL.loadtandemfull(FORCE_RELOAD);
pause(0.1); % So everything displays nicely

%% DATA PROCESSING - FREQUENCY DIFFERENCE
% Shows how the frequency is not always rigorously equal between the two sets

printsection('1. Frequency difference');

% Gather all frequencies in array for plotting
freqs_3cm = getallfreqs(RL.Tand0012_3);
freqs_5cm = getallfreqs(RL.Tand0012_5);
freqs_10cm = getallfreqs(RL.Tand0012_10);

% Figure
figname = 'PROCESSING - Frequency Difference';
close(findobj('type', 'figure', 'name', figname));
figure('Name', figname);
hold on;

% Measured frequencies
freqplot(freqs_3cm, 'o', Uliegecolors.blueGreen, Uliegecolors.blueGreenLight);
freqplot(freqs_5cm, 's', Uliegecolors.orange, Uliegecolors.yellowOrange);
freqplot(freqs_10cm, '^', Uliegecolors.violet, Uliegecolors.purple);
% Ideal match
freqRange = [0 5];
plot(freqRange, freqRange, '--k', 'linewidth', 0.5);
hold off;

% Plot style
legend('\dx = 3 cm', '\dx = 5 cm', '\dx = 10 cm', 'location', 'NorthWest');
legend('boxoff');
xlabel('$\freq_\front$ [Hz]');
ylabel('$\freq_\aft$ [Hz]');
axis equal;
xlim(freqRange);
ylim(freqRange);
setgca();
axis square;

% Save figure
figDir = '../figures/methodo/';
figName = 'freq-compa_front_aft.tex';
condsave2tikz(SAVE_FIGURES, [], [figDir, figName], '\normalsize');

% Mean error w.r.t front set for all distances
fprintf(' 3 cm | Mean F_error = %0.2f %%\n', meanerror(freqs_3cm) * 100);
fprintf(' 5 cm | Mean F_error = %0.2f %%\n', meanerror(freqs_5cm) * 100);
fprintf('10 cm | Mean F_error = %0.2f %%\n', meanerror(freqs_10cm) * 100);

%% DATA PROCESSING - FREQUENCY VARIATION
% Shows how the frequency variates during a single run

printsection('2. Frequency variation');

RESULTS_DATA = RL.Tand0012_3; % Data set to use
TEST_TO_PLOT = 19; % Plot a single test. Use [] to plot all

% Make all figures
NB_BINS = 15;
for iTest = 1:numel(RESULTS_DATA)

    if ~RESULTS_DATA(iTest).rejected && (iTest == TEST_TO_PLOT || isempty(TEST_TO_PLOT))

        % Impossible to use probability normalization in MATLAB < 2023, so workaround
        [countsF, binValuesF] = histcounts(1 ./ RESULTS_DATA(iTest).PeriodsF.rawT, NB_BINS);
        [countsA, binValuesA] = histcounts(1 ./ RESULTS_DATA(iTest).PeriodsA.rawT, NB_BINS);
        countsF_pc = 100 * countsF / sum(countsF);
        countsA_pc = 100 * countsA / sum(countsA);

        % Figure - BOTH SETS
        figname = sprintf('PROCESSING - Frequency Variation (both) %d', iTest);
        prevFigname = sprintf('PROCESSING - Frequency Variation (both) %d', iTest - 1);
        close(findobj('type', 'figure', 'name', prevFigname));
        figure('Name', figname);

        % Make histogram
        hAx1 = histogram('BinEdges', binValuesF, 'BinCounts', countsF_pc, ...
                         'DisplayStyle', 'stairs', ...
                         'LineWidth', 1.8, ...
                         'EdgeColor', Uliegecolors.blueGreen);
        hold on;
        hAx2 = histogram('BinEdges', binValuesA, 'BinCounts', countsA_pc, ...
                         'DisplayStyle', 'stairs', ...
                         'LineWidth', 1.8, ...
                         'EdgeColor', Uliegecolors.orange);
        hold off;

        % Legend
        text(2.7, 45, ...
             '\textcolor{ulgBlueGreen}{Front}', ...
             'HorizontalAlignment', 'left');
        text(2.72,  27, ...
             '\textcolor{ulgOrange}{Aft}', ...
             'HorizontalAlignment', 'left');

        % Plot style
        setgca();
        figTitle = RESULTS_DATA(iTest).strcond('title', 'meanfreq');
        xlabel('$\freq$ [Hz]');
        ylim([0, 50]);
        yticks(0:25:50);
        yticklabels({'0\,%', '25\,%', '50\,%'});
        axis square; % Ensure we have the same aspect ratio as compa_freq graph to plot side by side
        % Save figure
        figDir = '../figures/methodo/';
        figName = sprintf('freq-distrib(both)-D%d_V%0.1f_F%0.2f.tex', ...
                          RESULTS_DATA(iTest).TestCond.spacing(1) * 100, ...
                          RESULTS_DATA(iTest).TestCond.airspeed, ...
                          RESULTS_DATA(iTest).meanFreq);
        condsave2tikz(SAVE_FIGURES, figTitle, [figDir, figName], '\normalsize');

        fprintf('Number of flapping cycles in hsitogram: %d\n', ...
                numel(RESULTS_DATA(iTest).PeriodsF.rawT));

        if isempty(TEST_TO_PLOT)
            pause; % Do not show them all at once
        end

    end

end

%% DATA PROCESSING - INERTIA/ADDED MASS CONTRIBUTION
% This shows the difference between the raw vertical force, the inertia contribution (obtained from
% wind-off conditions) and the resulting aerodynamic loads.
% The period is offset so it starts at the end of the downstroke, rather than when we are at the
% middle of upstroke.

printsection('3. Inertia removal');

RESULTS_DATA = RL.Tand0012_3; % Data set to use
TEST_TO_PLOT = [5:5:29] - 1; % Plot a single test. Use [] to plot all
DEBUG_PLOTS = false; % Additional plots for debugging
SAVE_FIGURES = false;

maxInertia = [];
maxRaw = [];
maxAero = [];
testFreq = [];

% Make all figures (FRONT PERIOD ONLY)
for iTest = 1:numel(RESULTS_DATA)

    if ~RESULTS_DATA(iTest).rejected && (isempty(TEST_TO_PLOT) || any(iTest == TEST_TO_PLOT))

        figTitle = RESULTS_DATA(iTest).strcond('title', 'meanfreq');
        fprintf('Plotting test %2d -- %s\n', iTest, figTitle);

        % Recreate raw data by redoing normalization and averaging from the base table
        raw_res = TestResult.prunedcopy(RESULTS_DATA(iTest), 'none');
        raw_res.PeriodsF.normalize();
        raw_res.PeriodsA.normalize();
        raw_res.PeriodsF.averageall();
        raw_res.PeriodsA.averageall(); % Make sure we also do if with aft in case we use it later

        % Normalize time vector so it is t/T [-]
        timevect = RESULTS_DATA(iTest).PeriodsF.Avg.MedianData.time;
        timevect = timevect / timevect(end); % Normalize to have t/T [-]

        % Reorganize data so it start with upstroke
        rawAvgF = raw_res.PeriodsF.reorganize('upstroke');
        rawAvgF = rawAvgF.MedianData;
        aeroAvgF = RESULTS_DATA(iTest).PeriodsF.reorganize('upstroke');
        aeroAvgF = aeroAvgF.MedianData;
        inertiaAvgF = [];
        if ~isempty(RESULTS_DATA(iTest).inertia_idx)
            inertiaAvgF = RESULTS_DATA(RESULTS_DATA(iTest).inertia_idx) ...
                .PeriodsF.reorganize('upstroke');
            inertiaAvgF = inertiaAvgF.MedianData;
        end

        % Get index of separation between upstroke and downstroke
        [~, ustkIdx] = min(rawAvgF.angle);
        [~, dstkIdx] = max(rawAvgF.angle);

        if DEBUG_PLOTS
            % In theory aeroAvgF.angle = rawAvgF.angle, as we do not remove inertia from angle
            figure('Name', 'DEBUG - Angle vs Time');
            inertiaplot(rawAvgF, inertiaAvgF, aeroAvgF, timevect, 'angle');
            xlabel('t/T [-]');
            ylabel('Flapping Angle [deg]');
            title(figTitle);
            grid on;
            legend;

            figure('Name', 'DEBUG - F_ZU vs Angle');
            hold on;
            inertiaplot(rawAvgF, inertiaAvgF, aeroAvgF, anglevect, 'Fz');
            xlabel('Flapping Angle [deg]');
            ylabel('F_ZU [N]');
            title(figTitle);
            grid on;
            legend;
            pause;
            continue
        end

        % Figure - ONLY FRONT WING
        figname = sprintf('PROCESSING - Inertia contribution (front) %d', iTest);
        prevFigname = sprintf('PROCESSING - Inertia contribution (front) %d', iTest - 1);
        close(findobj('type', 'figure', 'name', prevFigname));
        figure('Name', figname);

        % Plot patch to highlight upstroke/downstroke
        patchX = [0, timevect(dstkIdx), timevect(dstkIdx), 0];
        patchY = 2 * max(rawAvgF.Fz) * [-1, -1, 1, 1];
        patch(patchX, patchY, Uliegecolors.beigeLight, 'EdgeColor', 'none', 'FaceAlpha', 0.3);
        text(timevect(dstkIdx) / 2, -1.78, ...
             'Upstroke', ...
             'HorizontalAlignment', 'center', ...
             'VerticalAlignment', 'bottom');
        text(timevect(dstkIdx) + ...
             (1 - (timevect(dstkIdx))) / 2, -1.78, ...
             'Downstroke', ...
             'HorizontalAlignment', 'center', ...
             'VerticalAlignment', 'bottom');

        % Plot lines
        hPlots = inertiaplot(rawAvgF, inertiaAvgF, aeroAvgF, timevect, 'Fz');

        % Legend
        % legend(hPlots, ...
        %     '$\vinf= 7.7$ m/s', '$\vinf = 0$ m/s', 'Difference', ...
        %     'location', 'SouthEast', 'Interpreter', 'Latex');
        % legend('boxoff');
        text(0.60, 1.32, ...
             '\textcolor{ulgBlueGreen}{Wind-on}', ...
             'HorizontalAlignment', 'right');
        text(0.80, 0.80, ...
             '\textcolor{ulgRed}{Wind-off}', ...
             'HorizontalAlignment', 'right', ...
             'VerticalAlignment', 'top');
        text(0.71, 0.10, ...
             '\textcolor{ulgYellowOrange}{Difference}', ...
             'HorizontalAlignment', 'center', ...
             'VerticalAlignment', 'top');

        % Plot style
        setgca();
        figTitle = RESULTS_DATA(iTest).strcond('title', 'meanfreq');
        xlabel('$t/T$ [-]');
        ylabel('$\fZ$ [N]');
        ylim([-1.8, 1.8]);
        xticks(0:0.25:1);
        axis square; % Ensure we have the same aspect ratio as compa_freq graph to plot side by side
        % Save figure
        figDir = '../figures/methodo/';
        figName = sprintf('inertia-D%d_V%0.1f_F%0.2f.tex', ...
                          RESULTS_DATA(iTest).TestCond.spacing(1) * 100, ...
                          RESULTS_DATA(iTest).TestCond.airspeed, ...
                          RESULTS_DATA(iTest).meanFreq);
        condsave2tikz(SAVE_FIGURES, figTitle, [figDir, figName], '\normalsize');

        % Print some information in console about current case
        nInertiaPer = numel(RESULTS_DATA(RESULTS_DATA(iTest).inertia_idx).PeriodsF.Normalized);
        maxInertia = [maxInertia, max(abs(inertiaAvgF.Fz))];
        maxRaw = [maxRaw, max(abs(rawAvgF.Fz))];
        maxAero = [maxAero, max(abs(aeroAvgF.Fz))];
        testFreq = [testFreq, RESULTS_DATA(iTest).meanFreq];

        fprintf('Number of inertia periods: %d\n', nInertiaPer);
        fprintf('  Max Raw_Fz: %0.2f\n  Max In_Fz: %0.2f\n  Diff In_Fz %0.3f \n', ...
                maxRaw(end),  maxInertia(end), (maxInertia(end) - maxRaw(end)) / maxRaw(end));

        %         RESULTS_DATA(RESULTS_DATA(iTest).inertia_idx).PeriodsF.plotaverage('Fz', true)

        if isempty(TEST_TO_PLOT)
            pause(0.1); % Do not show them all at once
        end

    end

end

pause(0.5);
figure('Name', 'Verif');
hold on;
plot(testFreq, maxRaw, 'DisplayName', 'Max Raw');
plot(testFreq, maxInertia, 'DisplayName', 'Max Inertia');
plot(testFreq, maxAero, 'DisplayName', 'Max Aero');
plot(testFreq, maxInertia ./ maxAero, '--', 'DisplayName', 'Ratio Inertia/Aero');
hold off;
legend;

%% MAINTENANCE / CHORES

% Copy all images created here to the thesis directory for inclusion in latex document
if SAVE_FIGURES
    copydir2dir(MATLAB_FIG_DIR, THESIS_FIG_DIR);
end

%% ================================== HELPER FUNCTIONS =============================================

function freqTable = getallfreqs(TestResults)
    % GETALLFREQS Output the true frequency of the front and aft wing for each testcase

    % Initialize as NaN table so rejected data are easily discarded
    freqTable = nan(numel(TestResults), 2);

    for i = 1:numel(TestResults)
        if ~TestResults(i).rejected
            freqTable(i, 1:2) = TestResults(i).trueFreq;
        end
    end
end

function freqplot(freqArray, markerType, edgeColor, faceColor)
    % FREQPLOT Plot frequencies of the aft set against the ones of the front set

    plot(freqArray(:, 1), freqArray(:, 2), markerType, ...
         'MarkerFaceColor', faceColor, ...
         'MarkerEdgeColor', edgeColor);

end

function meanError = meanerror(array2d)
    % MEANERROR Mean error calculation bewteen [M, 2] arrays

    relError = (array2d(:, 2) - array2d(:, 1)) ./ array2d(:, 1);

    meanError = mean(abs(relError), 'omitnan'); % Remove NaN values from mean calculation
end

function hPlots = inertiaplot(rawTable, inertiaTable, aeroTable, xVect, yField)
    % Plot the comparison beteween total, inertia and aerodynamic contributions

    LW = 0.6; % Line Width

    hold on;
    hPlots(1) = plot(xVect, rawTable.(yField), ...
                     'color', Uliegecolors.blueGreen, ...
                     'Linewidth', LW, ...
                     'DisplayName', 'total');
    if ~isempty(inertiaTable)
        hPlots(2) = plot(xVect, inertiaTable.(yField), ...
                         'color', Uliegecolors.red, ...
                         'Linewidth', LW,  ...
                         'DisplayName', 'inertia');
    end
    hPlots(3) = plot(xVect, aeroTable.(yField), ...
                     'color', Uliegecolors.yellowOrange, ...
                     'Linewidth', 2 * LW, ...
                     'DisplayName', 'aero');
    hold off;

end
