function ThisTest = processall(varargin)
    % PROCESSALL Process all data file listed in the identification files `idFile`.
    %   This function is the main processing tool for all the experimental data related to the
    %   MechaRaptor.
    %
    %   It parses the idFile, then opens the individual data recordings for each test case, clean
    %   and filter them, and finally synchronize the data points between the various sources of data
    %   for further analysis.
    %   The TestCase objects created for each test condition are then saved to avoid unnecessary
    %   re-processing.
    % -----
    %
    % Syntax:
    %   PROCESSALL() prompts the user for one identification file to process.
    %
    %   PROCESSALL(idFile) process all test cases found in the identification file provided.
    %
    %   PROCESSALL(..., 'autosave', true) save TestCase objects automatically (default = true).
    %
    %   PROCESSALL(..., 'prune', all) prune original data to keep low size. Options are
    %       - 'none': does not prune anything
    %       - 'raw' : only prune ArduDat.Raw and TunnelDat.Raw
    %       - 'all' : (default) prune entirely ArduDat and TunnelDat once data are synced and merded
    %
    %   PROCESSALL(..., 'debug', true) print debug information (default = false).
    %
    %   PROCESSALL(..., 'quiet', true) do not print info about current action (default = false).
    %
    % Inputs:
    %   idFile: Identification file (string or char vector).
    %
    % Output:
    %   TestCase objects saved as mat file in ../results/
    %
    % See also: TestCase, FileLoader.
    %
    % ----------------------------------------------------------------------------------------------
    % FIXME (H): What about static tests with no Arduino records?
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    narginchk(0, 9); % FileId + 4 options with their values

    % Cleanup, imports and environment setting
    clearvars -except varargin;
    close all;
    clc;
    addpath(genpath('.'));
    setcwdhere(); % Make sure we are always in the proper current working directory

    % Defaults and constants
    WT_SAMPLING_RATE = 1e3;      % Sampling rate of the Wind Tunnel data;
    WT_CUTOFF_MULTIPLIER = 1.5;  % Multiplier for the theorical frequency to use as a cutoff, [-]

    % Parse inputs properly
    [idFiles, autosave, prune, debug, quiet] = parseinputs(varargin{:});

    % Process each identification table
    for file = idFiles

        % Get ID file information load it
        [fpath, fname] = fileparts(file);
        fpath = fileparts(fpath); % (FIXME: hacky) Test campaign main directory
        conditionalprint(~quiet || debug, sprintf("Processing ID file %s\n", fname));
        conditionalprint(~quiet || debug, "**********************************\n");
        idTable = loadidfile(file{:});

        AllTests = TestCase; % Re-initialize Objects for each file

        for iTest = 1:size(idTable, 1)

            conditionalprint(~quiet || debug, ...
                             sprintf("Processing: Dx= %.1fcm, Speed= %.1fm, Freq= %.2fHz\n", ...
                                     idTable(iTest, :).dstX * 100, idTable(iTest, :).airspeed, ...
                                     idTable(iTest, :).target_freq));

            % Create test case objects
            [arduFile, tunnelFile] = getdatafilename(fpath, idTable(iTest, :));
            ThisTest = TestCase([idTable(iTest, :).dstX, 0, 0], ...
                                idTable(iTest, :).airspeed, ...
                                idTable(iTest, :).target_freq, ...
                                idTable(iTest, :).temperature,  ...
                                idTable(iTest, :).pressure, ...
                                arduFile, tunnelFile);

            % Skip processing if data should be rejected
            if strfind(idTable.comments{iTest, :}, 'REJECT')
                ThisTest.rejected = true;
                AllTests(iTest) = ThisTest;
                warning(['DATA REJECTED as indicated in comments: ', idTable.comments{iTest, :}]);
                continue
            end

            % Set proper data information
            ThisTest.TunnelDat.updatesampling(WT_SAMPLING_RATE);

            % Clean and calibrate individual data sources
            conditionalprint(~quiet || debug, "Cleaning wind tunnel data\n");
            ThisTest.TunnelDat.cleandata(WT_CUTOFF_MULTIPLIER * idTable(iTest, :).target_freq);

            conditionalprint(~quiet || debug, "Cleaning arduino data\n");
            ThisTest.ArduDat.cleandata();

            % Sync and merge the two data sources
            conditionalprint(~quiet || debug, "Syncing and merging data\n");
            ThisTest.mergedata();

            if debug
                % Plots
                figname = 'Process (DEBUG) - Angles';
                close(findobj('type', 'figure', 'name', figname));

                figure('Name', figname);
                axF = subplot(1, 2, 1); % Front wing
                hold on;
                plot(ThisTest.ArduDat.Raw.angleFrontLeft);
                plot(ThisTest.ArduDat.Raw.angleFrontRight);
                plot(ThisTest.ArduDat.Cleaned.angleF);
                legend('Left', 'Right', 'Cleaned');
                grid on;

                axA = subplot(1, 2, 2); % Front wing
                hold on;
                plot(ThisTest.ArduDat.Raw.angleAftLeft);
                plot(ThisTest.ArduDat.Raw.angleAftRight);
                plot(ThisTest.ArduDat.Cleaned.angleA);
                legend('Left', 'Right', 'Cleaned');
                grid on;

                linkaxes([axF, axA], 'xy');

                ThisTest.getfreqspectrum(true); % Frequency Spectrum
                pause;

            end

            % Always prune raw data and only keep merged results
            ThisTest.clearall();
            ThisTest.saveNameFromTestID(file, []);

            % Multi-dimensional object for easier processing later on
            AllTests(iTest) = ThisTest;
            conditionalprint(~quiet || debug, "--------------------\n");

        end

        fprintf('===========================================================================\n\n');

        if autosave
            conditionalprint(~quiet || debug, "Saving all TestCase processed data\n");

            AllTests(1).saveNameFromTestID(file, []);
            saveFileName = AllTests(1).saveFileName;

            saveobjects(AllTests, saveFileName);
        end

    end

end

% --------------------------------------------------------------------------------------------------
function [fileNames, autosave, prune, debug, quiet] = parseinputs(varargin)
    % PARSEINPUTS Parse function input and return cleaned inputs and options

    % Default and constants
    DATA_DIR = '../data'; % Location of the data dir w.r.t. current file
    FILE_EXT = {'*.csv'; '*.txt'}; % Accepted file extensions
    PRUNE_OPTS = {'none', 'raw', 'all'}; % Pruning options
    DEF_PRUNE = 'all';

    % Validation functions
    valLogi = @(x) validateattributes(x, {'logical'}, {'scalar'});

    % Parse inputs
    p = inputParser;
    addOptional(p, 'files', []);
    addParameter(p, 'autosave', true, valLogi);
    addParameter(p, 'prune', DEF_PRUNE, @(x) any(validatestring(x, PRUNE_OPTS)));
    addParameter(p, 'debug', false, valLogi);
    addParameter(p, 'quiet', false, valLogi);
    parse(p, varargin{:});
    tmpFiles = string(p.Results.files);
    autosave = p.Results.autosave;
    prune = p.Results.prune;
    debug = p.Results.debug;
    quiet = p.Results.quiet;

    % Get proper file names
    fileNames = getfiles(DATA_DIR, FILE_EXT, tmpFiles);

end

function [arduFile, tunnelFile] = getdatafilename(idFileName, idTableLine)
    % GETDATAFILENAME Return entire data file name

    arduFile = fullfile(idFileName, 'arduinoData', idTableLine.arduinoData{:});
    tunnelFile = fullfile(idFileName, 'windtunnelData', idTableLine.windtunnelData{:});
end
