function [avgTable, lowTable, highTable] = averagetablestruct(tableStruct, syncTime, ...
                                                              interval, avgType)
    % AVERAGETABLESTRUCT Average a structure of similar tables.
    %   This function takes a structure made of tables of equal size and average all the data
    %   together either using 'mean' or 'median'. It then outputs the mean/median values in a single
    %   table, and the lower/higher percentiles or standard deviations in two other tables.
    % -----
    %
    % Syntax:
    %   TODO.
    %
    % Inputs:
    %   tableStruct : strcture of tables of equal dimensions
    %   syncTime    : synchronize all time data (needed for Periods average)
    %   interval    : percentile or number of standard deviations from mean
    %   avgType     : type of averaging ('median' or 'mean')
    %
    % Output:
    %   avgTable    : table with averaged data (median or mean)
    %   lowPcTable  : table with lower values (percentile or mean - interval*std)
    %   highPcTable : table with higher values (percentile or mean + interval*std)
    %
    % See also: Periods, TestCase.

    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input checks
    if nargin < 3
        interval = 25;
    end
    if nargin < 4
        avgType = 'median';
    end
    avgType = validatestring(avgType, {'mean', 'median'}, mfilename(), 'avgType', 3);

    % --- Syncing time so all time vectors are the same
    if syncTime
        time = tableStruct(1).Data.time - tableStruct(1).Data.time(1);
    end

    % --- Averaging tables

    % Create a dummy 3D table with all periods data
    dummy = zeros([size(tableStruct(1).Data), 3]);
    for iPer = 1:numel(tableStruct)
        dummy(:, :, iPer) = table2array(tableStruct(iPer).Data);
    end

    % Average tables
    switch avgType
        case 'mean'
            avgDummy = mean(dummy, 3);
            stdDummy = std(dummy, 0, 3);
            lowDummy = avgDummy - interval .* stdDummy;
            highDummy = avgDummy + interval .* stdDummy;
        case 'median'
            avgDummy = median(dummy, 3);
            pctileDummy = prctile(dummy, [interval, 100 - interval], 3);
            lowDummy = pctileDummy(:, :, 1);
            highDummy = pctileDummy(:, :, 2);
    end

    % Ouput
    avgTable = array2table(avgDummy);
    avgTable.Properties.VariableNames = tableStruct(1).Data.Properties.VariableNames;

    lowTable = array2table(lowDummy);
    lowTable.Properties.VariableNames = tableStruct(1).Data.Properties.VariableNames;

    highTable = array2table(highDummy);
    highTable.Properties.VariableNames = tableStruct(1).Data.Properties.VariableNames;

    if syncTime
        avgTable.time = time;
        lowTable.time = time;
        highTable.time = time;
    end

end
