function fullFileNames = getfiles(lookupDir, fileFormat, inputFileNames)
    % GETFILES Get files from user input or UI prompt.
    %   This function can be used to load multiple raw test data files or already processed files
    %   for further analsyis.
    % -----
    %
    % Syntax:
    %   frac = fracpart(x) returns the fractional part of real number x.
    %
    % Inputs:
    %   x : An real number (or array of real numbers).
    %
    % Output:
    %   frac : the fractional part of the input number(s).
    %
    % Example:
    %   fracpart(1.38) % returns 0.38
    %   fracpart([-4.2, 3.8]) % returns [-0.2, 0.8]

    % ----------------------------------------------------------------------------------------------
    % Source: https://en.wikipedia.org/wiki/Fractional_part
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input checks
    % Check lookup dir is single string or char vect
    % Check fileFormat is char vector or cell or char vectors or strings

    if nargin < 3
        inputFileNames = [];
    end
    % Check fileNames = string list

    % --- Get all exsisting files

    if isempty(inputFileNames)
        % Prompt user for files when no input
        fullFileNames = promptforfiles(lookupDir, fileFormat);
    else
        % Check if input names correspond to existing files
        fullFileNames = cell(size(inputFileNames));
        for i = 1:numel(inputFileNames)
            dummy = dir(inputFileNames(i));
            if isempty(dummy)
                warning('getfiles:fileNotFound', ...
                        'The file ''%s'' was not found.', inputFileNames(i));
            else
                for j = 1:numel(dummy)
                    fullFileNames{i, j} = fullfile(dummy(j).folder, dummy(j).name);
                end
            end
        end

        % Format output string list, remove empty/missing lines
        fullFileNames = reshape(fullFileNames, 1, []);
        fullFileNames = rmmissing(fullFileNames(~cellfun('isempty', fullFileNames)));
        fullFileNames = string(fullFileNames);

        % Warn user and prompt for selection of proper input files
        if isempty(fullFileNames)
            warning('getfiles:noFileFound', ...
                    'No valid file was found. Please select an actual file.');

            fullFileNames = promptforfiles(lookupDir, fileFormat);
        end
    end
end

function fullFileNames = promptforfiles(lookupDir, fileFormat)
    % PROMPTFORFILES Prompt user for inuput files

    [filenames, filepaths] = uigetfile(fileFormat, 'Select all files required for analysis', ...
                                       lookupDir, 'MultiSelect', 'on');
    if isa(filenames, 'double') % uigetfile return 0 if user canceled
        error('getfiles:cancelSelection', ...
              ['File selection cancelled by user.\n'...
               'Please select or enter valid file.']);
    end
    filenames = string(filenames);
    filepaths = repmat(string(filepaths), 1, length(filenames));
    fullFileNames = fullfile(filepaths, filenames);
end
