function idTable = loadidfile(idFile)
    % LOADID Load the identification file into a table.
    %   This function parse the headerlines and then import everything in a table using Matlab's
    %   readtable function.
    %
    % Note
    %   The identification table must obey to the following scheme
    %       # Header line 1
    %       # ... (any number of header lines)
    %       # Header last line
    %       DataName1, DataName2, ..., DataNameN
    %       DataUnit1, DataUnit2, ..., DataUnitN
    %       Data1, Data2, ..., Data3
    %
    % Note: The header lines must start with a '#' symbol
    % -----
    %
    % Syntax:
    %   idTable = LOADIDFILE(idFile) load the identification file into a Matlab table.
    %
    % Inputs:
    %   idFile: Identification file
    %
    % Outputs:
    %   idTable: Table with the identification data
    %
    % See also: readtable.

    % ----------------------------------------------------------------------------------------------
    % TODO (L): Check for inconsistencies in the table (in terms of speed, temperature, etc)
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2023 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Defaults and constants
    HEADER_STYLE = '#';

    % Determine where the data starts
    headerLines = calcheaderlines(idFile, HEADER_STYLE);

    % Tweak options before import
    opts = detectImportOptions(idFile);
    opts.VariableNamesLine = headerLines + 1;
    opts.VariableUnitsLine = headerLines + 2;
    opts.DataLines = [headerLines + 3 Inf];

    % Import
    idTable = readtable(idFile, opts);

end

function headerLines = calcheaderlines(file, headerStyle)
    % CALCHEADERLINES Calculate number of header lines in a file

    fileID = fopen(file);

    headerLines = 0;
    inHeader = true;

    while inHeader
        line = textscan(fileID, '%s', 1, 'Delimiter', {'\n', '\r'});
        line = char(line{:});

        if regexp(line, ['^\﻿*\s*', headerStyle])
            headerLines = headerLines + 1;
        else
            inHeader = false;
        end

    end

    fclose(fileID);
end
