function [phaseAngle] = phasediff(time1, time2, period)
    % PHASEDIFF Calculate the phase angle between two times of a signal.
    %   This function returns a phase angle in the [-pi; pi] interval.
    %   It is mostly used to determine the phase difference between the front and the aft wings
    %   angles, based on the front wing period.
    % -----
    %
    % Syntax:
    %   phaseAngle = phasediff(time1, time2, period) Calculates the phase angle between the
    %   reference time (time1) and a second time (time2), based on a reference period.
    %
    % Inputs:
    %   time1  : the reference time
    %   time2  : the time to get the phase difference with the reference
    %   period : the reference period
    %
    % Output:
    %   phaseAngle : the phase angle, between [-pi; pi];
    %
    % See also: plot.

    % ----------------------------------------------------------------------------------------------
    % Source: https://en.wikipedia.org/wiki/Phase_(waves)
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % Input checks
    validateattributes(time1, {'numeric'}, {'real'}, mfilename(), 'time1', 1);
    validateattributes(time2, {'numeric'}, {'real'}, mfilename(), 'time2', 2);
    validateattributes(period, {'numeric'}, {'real'}, mfilename(), 'period', 3);

    % Phase angle
    phaseAngle = 2 * pi * (fracpart((time2 - time1) / period + 1 / 2) - 1 / 2);

end
