function saveobjects(Objects, saveFileName)
    % SAVEOBJECTS Save multiple objects in a single MAT-file.
    %   This function saves multiple objects in a easy-to-load file.
    % -----
    %
    % Syntax:
    %   saveobjects(saveFileName, Objects) Save all objects in `Objects` in a MAT-file corresponding
    %   to `saveFileName`. If the directory indicated in `saveFileName` does not exist, it will be
    %   created on the fly.
    %
    % Inputs:
    %   Objects      : Objects to save in MAT-file. Can have multiple dimension.
    %   saveFileName : (optional) File name for the saved data. Default is the Object name.
    %
    % Output:
    %   MAT-file with the Objects as data in the location specified by `saveFileName`.
    %
    % Example:
    %   saveobjects(TestCase, 'myresults') -> create myresults.mat that contains Objects
    %
    % See also: TestCase, TestResult.
    %
    % ----------------------------------------------------------------------------------------------
    % (c) Copyright 2022-2024 University of Liege
    % Author: Thomas Lambert <t.lambert@uliege.be>
    % ULiege - Aeroelasticity and Experimental Aerodynamics
    % MIT License
    % Repo: https://gitlab.uliege.be/mecharaptor/data-processing
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % --- Input checks
    if nargin < 2
        saveFileName = inputname(1);
    end

    % --- Defaults and constants
    OVERWRITE = true; % [FIXME] Should use input parser ideally

    % --- Save Objects to file

    % Get directory and proper filename
    [saveDir, ~] = fileparts(saveFileName);
    if numel(Objects) > 1 && ~endsWith(saveFileName, '_ALL')
        saveFile = strcat(saveFileName, "_ALL.mat");
    else
        saveFile = saveFileName;
    end
    chkdir(saveDir);

    % Save file using structure so it is easier to load
    if ~exist(saveFile, 'file') || OVERWRITE
        tempFile.Objects = Objects;
        save(saveFile, '-struct', 'tempFile', '-v7.3');
    end

end
